"""Radarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from .base import BaseModel
from .request_common import (
    _Common3,
    _Common5,
    _Common9,
    _CommonAttrs,
    _MetadataFields,
    _Quality,
    _Ratings,
    _SelectOption,
)


@dataclass(init=False)
class _RadarrMovieFileMediaInfo(_CommonAttrs):
    """Radarr movie file media attributes."""

    audioAdditionalFeatures: str


@dataclass(init=False)
class _RadarrMovieImages(_Common5):
    """Radarr movie images attributes."""

    remoteUrl: str


@dataclass(init=False)
class _RadarrDatabaseRating(_Ratings):
    """Radarr databade rating attributes."""

    type: str


@dataclass(init=False)
class _RadarrMovieRatings(BaseModel):
    """Radarr movie ratings attributes."""

    imdb: type[_RadarrDatabaseRating] = field(default=_RadarrDatabaseRating)
    metacritic: type[_RadarrDatabaseRating] = field(default=_RadarrDatabaseRating)
    rottenTomatoes: type[_RadarrDatabaseRating] = field(default=_RadarrDatabaseRating)
    tmdb: type[_RadarrDatabaseRating] = field(default=_RadarrDatabaseRating)

    def __post_init__(self):
        self.imdb = _RadarrDatabaseRating(self.imdb)
        self.metacritic = _RadarrDatabaseRating(self.metacritic)
        self.rottenTomatoes = _RadarrDatabaseRating(self.rottenTomatoes)
        self.tmdb = _RadarrDatabaseRating(self.tmdb)


@dataclass(init=False)
class _RadarrMovieCollection(BaseModel):
    """Radarr movie collection attributes."""

    images: list[_RadarrMovieImages] | None = None
    name: str
    tmdbId: int

    def __post_init__(self):
        self.images = [_RadarrMovieImages(image) for image in self.images or []]


@dataclass(init=False)
class _RadarrMovieFields(_MetadataFields, _SelectOption):
    """Radarr movie fields attributes."""


@dataclass(init=False)
class _RadarrCommon(BaseModel):
    """Radarr indexers attributes."""

    fields: list[_RadarrMovieFields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    name: str

    def __post_init__(self):
        self.fields = [_RadarrMovieFields(field) for field in self.fields or []]


@dataclass(init=False)
class _RadarrMovieSpecifications(_RadarrCommon):
    """Radarr movie specifications attributes."""

    negate: bool
    required: bool


@dataclass(init=False)
class _RadarrMovieCustomFormats(_Common3):
    """Radarr movie custom formats attributes."""

    includeCustomFormatWhenRenaming: bool
    specifications: list[_RadarrMovieSpecifications] | None = None

    def __post_init__(self):
        self.specifications = [
            _RadarrMovieSpecifications(spec) for spec in self.specifications or []
        ]


@dataclass(init=False)
class _RadarrCommon2(BaseModel):
    """Radarr common attributes."""

    id: int
    minimumAvailability: str
    qualityProfileId: int


@dataclass(init=False)
class _RadarrCommon3(_Common9):
    """Radarr common attributes."""

    collection: type[_RadarrMovieCollection] = field(default=_RadarrMovieCollection)
    digitalRelease: datetime
    images: list[_RadarrMovieImages] | None = None
    inCinemas: datetime
    physicalRelease: datetime
    ratings: type[_RadarrMovieRatings] = field(default=_RadarrMovieRatings)
    sortTitle: str
    status: str
    studio: str
    tmdbId: int
    website: str
    youTubeTrailerId: str

    def __post_init__(self):
        """Post init."""
        self.collection = _RadarrMovieCollection(self.collection)
        self.images = [_RadarrMovieImages(image) for image in self.images or []]
        self.ratings = _RadarrMovieRatings(self.ratings)


@dataclass(init=False)
class _RadarrMovieCommon(BaseModel):
    """Radarr movie common attributes."""

    edition: str
    id: int
    languages: list[_Common3] | None = None
    quality: type[_Quality] = field(default=_Quality)

    def __post_init__(self):
        self.languages = [_Common3(language) for language in self.languages or []]
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _RadarrMovieHistoryBlocklistBase(_RadarrMovieCommon):
    """Radarr movie history/blocklist attributes."""

    customFormats: list[_RadarrMovieCustomFormats] | None = None
    date: datetime
    movieId: int
    sourceTitle: str

    def __post_init__(self):
        super().__post_init__()
        self.customFormats = [
            _RadarrMovieCustomFormats(custForm) for custForm in self.customFormats or []
        ]


@dataclass(init=False)
class _RadarrNotificationMessage(BaseModel):
    """Radarr notification message attributes."""

    message: str
    type: str


@dataclass(init=False)
class _RadarrMovieHistoryData(BaseModel):
    """Radarr movie history data attributes."""

    downloadClient: str
    downloadClientName: str
    droppedPath: str
    fileId: int
    importedPath: str
    reason: str


@dataclass(init=False)
class _RadarrMovieAlternateTitle(BaseModel):
    """Radarr movie history alternate title attributes."""

    id: int
    language: type[_Common3] = field(default=_Common3)
    movieId: int
    sourceId: int
    sourceType: str
    title: str
    voteCount: int
    votes: int

    def __post_init__(self):
        """Post init."""
        self.language = _Common3(self.language)


@dataclass(init=False)
class _RadarrParsedMovieInfo(BaseModel):
    """Radarr parsed movie info attributes."""

    edition: str
    extraInfo: dict
    imdbId: str
    languages: list[_Common3] | None = None
    movieTitle: str
    movieTitles: list[str]
    originalTitle: str
    primaryMovieTitle: str
    quality: type[_Quality] = field(default=_Quality)
    releaseHash: str
    releaseTitle: str
    simpleReleaseTitle: str
    tmdbId: int
    year: int

    def __post_init__(self):
        """Post init."""
        self.languages = [_Common3(x) for x in self.languages or []]
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _RadarrCustomFormatsSpecsFields(BaseModel):
    """Radarr custom formats specifications fields attributes."""

    value: int


@dataclass(init=False)
class _RadarrCustomFormatsSpecs(BaseModel):
    """Radarr custom formats specifications attributes."""

    fields: type[_RadarrCustomFormatsSpecsFields] = field(
        default=_RadarrCustomFormatsSpecsFields
    )
    implementation: str
    negate: bool
    required: bool

    def __post_init__(self):
        """Post init."""
        self.fields = _RadarrCustomFormatsSpecsFields(self.fields)


@dataclass(init=False)
class _RadarrCustomFormats(BaseModel):
    """Radarr custom formats attributes."""

    includeCustomFormatWhenRenaming: bool
    name: str
    specifications: list[_RadarrCustomFormatsSpecs] | None = None

    def __post_init__(self):
        """Post init."""
        self.specifications = [
            _RadarrCustomFormatsSpecs(x) for x in self.specifications or []
        ]



"""Request Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes, line-too-long, too-many-lines
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from .base import BaseModel
from .const import ProtocolType


@dataclass(init=False)
class _SelectOption(BaseModel):
    """Select option attributes."""

    name: str
    order: int
    value: str | list[str] | int


@dataclass(init=False)
class _SelectOptionExtended(_SelectOption):
    """Select options extended attributes."""

    dividerAfter: bool


@dataclass(init=False)
class _Fields(_SelectOption):
    """Fields attributes."""

    advanced: bool
    errors: list
    helpText: str
    hidden: str
    label: str
    pending: bool
    selectOptions: list[_SelectOptionExtended] | None = None
    type: str
    warnings: list

    def __post_init__(self):
        self.selectOptions = [
            _SelectOptionExtended(opt) for opt in self.selectOptions or []
        ]


@dataclass(init=False)
class _Common(BaseModel):
    """Common attributes."""

    fields: list[_Fields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    name: str

    def __post_init__(self):
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class _Common2(BaseModel):
    """Common attributes."""

    downloadId: str
    eventType: str


@dataclass(init=False)
class _Common3(BaseModel):
    """Common attributes."""

    id: int
    name: str


@dataclass(init=False)
class _Common4(BaseModel):
    """Common attributes."""

    downloadClient: str
    downloadId: str
    estimatedCompletionTime: datetime
    indexer: str
    outputPath: str


@dataclass(init=False)
class _Common5(BaseModel):
    """Common attributes."""

    coverType: str
    url: str


@dataclass(init=False)
class _Common6(BaseModel):
    """Common attributes."""

    monitored: bool
    overview: str


@dataclass(init=False)
class _Common7(BaseModel):
    """Common attributes."""

    id: int
    indexer: str
    protocol: ProtocolType


@dataclass(init=False)
class _QualityInfo(_Common3):
    """Quality info attributes."""

    modifier: str
    resolution: int
    source: str


@dataclass(init=False)
class _Revision(BaseModel):
    """Revision attributes."""

    isRepack: bool
    real: int
    version: int


@dataclass(init=False)
class _Quality(BaseModel):
    """Quality attributes."""

    quality: type[_QualityInfo] = field(default=_QualityInfo)
    revision: type[_Revision] = field(default=_Revision)

    def __post_init__(self):
        """Post init."""
        self.quality = _QualityInfo(self.quality)
        self.revision = _Revision(self.revision)


@dataclass(init=False)
class _Common8(BaseModel):
    """Common attributes."""

    id: int
    protocol: ProtocolType
    quality: type[_Quality] = field(default=_Quality)
    size: int
    sizeleft: int
    status: str
    statusMessages: list[_StatusMessage] | None = None
    timeleft: str
    title: str
    trackedDownloadState: str
    trackedDownloadStatus: str


@dataclass(init=False)
class _Common9(BaseModel):
    """Common attributes."""

    certification: str
    genres: list[str]
    imdbId: str
    runtime: int
    title: str
    year: int


@dataclass(init=False)
class _CommonAttrs(BaseModel):
    """Common attributes."""

    audioBitrate: str
    audioChannels: float
    audioCodec: str
    audioLanguages: str
    audioStreamCount: int
    resolution: str
    runTime: str
    scanType: str
    subtitles: str
    videoBitDepth: int
    videoBitrate: int
    videoCodec: str
    videoDynamicRangeType: str
    videoFps: float


@dataclass(init=False)
class _CommandBody(BaseModel):
    """Command body attributes."""

    completionMessage: str
    isExclusive: bool
    isNewMovie: bool
    isTypeExclusive: bool
    lastExecutionTime: datetime
    lastStartTime: datetime
    name: str
    requiresDiskAccess: bool
    sendUpdatesToClient: bool
    suppressMessages: bool
    trigger: str
    updateScheduledTask: bool


@dataclass(init=False)
class _CustomFilterAttr(BaseModel):
    """Custom filter attributes."""

    key: str
    type: str
    value: list[str]


@dataclass(init=False)
class _MetadataFields(_Fields):
    """Metadata fields attributes."""

    section: str


@dataclass(init=False)
class _FilesystemFolder(BaseModel):
    """Filesystem folder attributes."""

    name: str
    path: str


@dataclass(init=False)
class _FilesystemDirectory(_FilesystemFolder):
    """Filesystem directory attributes."""

    lastModified: datetime
    size: int
    type: str


@dataclass(init=False)
class _ImportListCommon(_FilesystemFolder):
    """Import list common attributes."""

    configContract: str
    listOrder: int
    rootFolderPath: str


@dataclass(init=False)
class _LocalizationStrings(BaseModel):
    """Localization strings attributes."""

    About: str
    Absolute: str
    AcceptConfirmationModal: str
    Actions: str
    Activity: str
    Add: str
    AddCustomFormat: str
    AddDelayProfile: str
    AddDownloadClient: str
    Added: str
    AddedAuthorSettings: str
    AddedToDownloadQueue: str
    AddExclusion: str
    AddImportListExclusionHelpText: str
    AddIndexer: str
    AddingTag: str
    AddList: str
    AddListExclusion: str
    AddMissing: str
    AddMovie: str
    AddMovies: str
    AddMoviesMonitored: str
    AddNew: str
    AddNewItem: str
    AddNewMessage: str
    AddNewMovie: str
    AddNewTmdbIdMessage: str
    AddNotification: str
    AddQualityProfile: str
    AddRemotePathMapping: str
    AddRestriction: str
    AddRootFolder: str
    AddToDownloadQueue: str
    AdvancedSettingsHiddenClickToShow: str
    AdvancedSettingsShownClickToHide: str
    AfterManualRefresh: str
    Age: str
    Agenda: str
    AgeWhenGrabbed: str
    All: str
    AllAuthorBooks: str
    AllBooks: str
    AllExpandedCollapseAll: str
    AllExpandedExpandAll: str
    AllFiles: str
    AllMoviesHiddenDueToFilter: str
    AllMoviesInPathHaveBeenImported: str
    AllowAuthorChangeClickToChangeAuthor: str
    AllowedLanguages: str
    AllowFingerprinting: str
    AllowFingerprintingHelpText: str
    AllowFingerprintingHelpTextWarning: str
    AllowHardcodedSubs: str
    AllowHardcodedSubsHelpText: str
    AllResultsHiddenFilter: str
    AlreadyInYourLibrary: str
    AlternateTitles: str
    AlternateTitleslength1Title: str
    AlternateTitleslength1Titles: str
    AlternativeTitle: str
    Always: str
    AnalyseVideoFiles: str
    Analytics: str
    AnalyticsEnabledHelpText: str
    AnalyticsEnabledHelpTextWarning: str
    Announced: str
    AnnouncedMsg: str
    AnyEditionOkHelpText: str
    ApiKey: str
    APIKey: str
    ApiKeyHelpTextWarning: str
    AppDataDirectory: str
    AppDataLocationHealthCheckMessage: str
    Apply: str
    ApplyTags: str
    ApplyTagsHelpTexts1: str
    ApplyTagsHelpTexts2: str
    ApplyTagsHelpTexts3: str
    ApplyTagsHelpTexts4: str
    AptUpdater: str
    AreYouSureYouWantToDeleteFormat: str
    AreYouSureYouWantToDeleteThisDelayProfile: str
    AreYouSureYouWantToDeleteThisImportListExclusion: str
    AreYouSureYouWantToDeleteThisRemotePathMapping: str
    AreYouSureYouWantToRemoveSelectedItemFromQueue: str
    AreYouSureYouWantToRemoveSelectedItemsFromQueue: str
    AreYouSureYouWantToRemoveTheSelectedItemsFromBlocklist: str
    AreYouSureYouWantToResetYourAPIKey: str
    AsAllDayHelpText: str
    ASIN: str
    AudioFileMetadata: str
    AudioInfo: str
    AuthBasic: str
    Authentication: str
    AuthenticationMethodHelpText: str
    AuthForm: str
    Author: str
    AuthorClickToChangeBook: str
    AuthorEditor: str
    AuthorFolderFormat: str
    AuthorIndex: str
    AuthorNameHelpText: str
    Authors: str
    Automatic: str
    AutomaticallySwitchEdition: str
    AutomaticSearch: str
    AutoRedownloadFailedHelpText: str
    AutoUnmonitorPreviouslyDownloadedBooksHelpText: str
    AutoUnmonitorPreviouslyDownloadedMoviesHelpText: str
    AvailabilityDelay: str
    AvailabilityDelayHelpText: str
    Backup: str
    BackupFolderHelpText: str
    BackupIntervalHelpText: str
    BackupNow: str
    BackupRetentionHelpText: str
    Backups: str
    BeforeUpdate: str
    BindAddress: str
    BindAddressHelpText: str
    BindAddressHelpTextWarning: str
    Blocklist: str
    Blocklisted: str
    BlocklistHelpText: str
    BlocklistRelease: str
    BlocklistReleases: str
    Book: str
    BookAvailableButMissing: str
    BookDownloaded: str
    BookEditor: str
    BookFileCountBookCountTotalTotalBookCountInterp: str
    BookFileCounttotalBookCountBooksDownloadedInterp: str
    BookFilesCountMessage: str
    BookHasNotAired: str
    BookIndex: str
    BookIsDownloading: str
    BookIsDownloadingInterp: str
    BookIsNotMonitored: str
    BookList: str
    BookMissingFromDisk: str
    BookMonitoring: str
    BookNaming: str
    Books: str
    BooksTotal: str
    BookStudio: str
    BookTitle: str
    Branch: str
    BranchUpdate: str
    BranchUpdateMechanism: str
    BuiltIn: str
    BypassDelayIfHighestQuality: str
    BypassDelayIfHighestQualityHelpText: str
    BypassProxyForLocalAddresses: str
    Calendar: str
    CalendarOptions: str
    CalendarWeekColumnHeaderHelpText: str
    CalibreContentServer: str
    CalibreContentServerText: str
    CalibreHost: str
    CalibreLibrary: str
    CalibreMetadata: str
    CalibreNotCalibreWeb: str
    CalibreOutputFormat: str
    CalibreOutputProfile: str
    CalibrePassword: str
    CalibrePort: str
    CalibreSettings: str
    CalibreUrlBase: str
    CalibreUsername: str
    Cancel: str
    CancelMessageText: str
    CancelPendingTask: str
    CancelProcessing: str
    CantFindMovie: str
    Cast: str
    CatalogNumber: str
    CertificateValidation: str
    CertificateValidationHelpText: str
    Certification: str
    CertificationCountry: str
    CertificationCountryHelpText: str
    CertValidationNoLocal: str
    ChangeFileDate: str
    ChangeHasNotBeenSavedYet: str
    CheckDownloadClientForDetails: str
    CheckForFinishedDownloadsInterval: str
    ChmodFolder: str
    ChmodFolderHelpText: str
    ChmodFolderHelpTextWarning: str
    ChmodGroup: str
    ChmodGroupHelpText: str
    ChmodGroupHelpTextWarning: str
    ChooseAnotherFolder: str
    ChownGroup: str
    ChownGroupHelpText: str
    ChownGroupHelpTextWarning: str
    CleanLibraryLevel: str
    Clear: str
    ClickToChangeLanguage: str
    ClickToChangeMovie: str
    ClickToChangeQuality: str
    ClickToChangeReleaseGroup: str
    ClientPriority: str
    CloneCustomFormat: str
    CloneFormatTag: str
    CloneIndexer: str
    CloneProfile: str
    Close: str
    CloseCurrentModal: str
    CollapseMultipleBooks: str
    CollapseMultipleBooksHelpText: str
    Collection: str
    ColonReplacement: str
    ColonReplacementFormatHelpText: str
    Columns: str
    CompletedDownloadHandling: str
    Component: str
    Conditions: str
    Connect: str
    Connection: str
    ConnectionLost: str
    ConnectionLostAutomaticMessage: str
    ConnectionLostMessage: str
    Connections: str
    ConnectSettings: str
    ConnectSettingsSummary: str
    ConsideredAvailable: str
    ConsoleLogLevel: str
    Continuing: str
    ContinuingAllBooksDownloaded: str
    ContinuingMoreBooksAreExpected: str
    ContinuingNoAdditionalBooksAreExpected: str
    CopyToClipboard: str
    CopyUsingHardlinksHelpText: str
    CopyUsingHardlinksHelpTextWarning: str
    CouldNotConnectSignalR: str
    CouldNotFindResults: str
    Country: str
    CreateEmptyAuthorFolders: str
    CreateEmptyAuthorFoldersHelpText: str
    CreateEmptyMovieFolders: str
    CreateEmptyMovieFoldersHelpText: str
    CreateGroup: str
    Crew: str
    CurrentlyInstalled: str
    Custom: str
    CustomFilters: str
    CustomFormat: str
    CustomFormatHelpText: str
    CustomFormatJSON: str
    CustomFormats: str
    CustomFormatScore: str
    CustomFormatsSettings: str
    CustomFormatsSettingsSummary: str
    CustomFormatUnknownCondition: str
    CustomFormatUnknownConditionOption: str
    Cutoff: str
    CutoffFormatScoreHelpText: str
    CutoffHelpText: str
    CutoffUnmet: str
    Date: str
    Dates: str
    Day: str
    Days: str
    DBMigration: str
    Debug: str
    DefaultCase: str
    DefaultDelayProfile: str
    DefaultMetadataProfileIdHelpText: str
    DefaultMonitorOptionHelpText: str
    DefaultQualityProfileIdHelpText: str
    DefaultReadarrTags: str
    DefaultTagsHelpText: str
    DelayingDownloadUntilInterp: str
    DelayProfile: str
    DelayProfiles: str
    Delete: str
    DeleteBackup: str
    DeleteBackupMessageText: str
    DeleteBookFile: str
    DeleteBookFileMessageText: str
    DeleteCustomFormat: str
    Deleted: str
    DeleteDelayProfile: str
    DeleteDelayProfileMessageText: str
    DeletedMsg: str
    DeleteDownloadClient: str
    DeleteDownloadClientMessageText: str
    DeleteEmptyFolders: str
    DeleteEmptyFoldersHelpText: str
    DeleteFile: str
    DeleteFileLabel: str
    DeleteFilesHelpText: str
    DeleteFilesLabel: str
    DeleteHeader: str
    DeleteImportList: str
    DeleteImportListExclusion: str
    DeleteImportListExclusionMessageText: str
    DeleteImportListMessageText: str
    DeleteIndexer: str
    DeleteIndexerMessageText: str
    DeleteList: str
    DeleteListMessageText: str
    DeleteMetadataProfile: str
    DeleteMetadataProfileMessageText: str
    DeleteMovieFolderHelpText: str
    DeleteMovieFolderLabel: str
    DeleteNotification: str
    DeleteNotificationMessageText: str
    DeleteQualityProfile: str
    DeleteQualityProfileMessageText: str
    DeleteReleaseProfile: str
    DeleteReleaseProfileMessageText: str
    DeleteRestriction: str
    DeleteRestrictionHelpText: str
    DeleteRootFolder: str
    DeleteRootFolderMessageText: str
    DeleteSelectedBookFiles: str
    DeleteSelectedBookFilesMessageText: str
    DeleteSelectedMovie: str
    DeleteSelectedMovieFiles: str
    DeleteSelectedMovieFilesMessage: str
    DeleteTag: str
    DeleteTagMessageText: str
    DeleteTheMovieFolder: str
    DestinationPath: str
    DestinationRelativePath: str
    DetailedProgressBar: str
    DetailedProgressBarHelpText: str
    Details: str
    Development: str
    DigitalRelease: str
    Disabled: str
    DiscCount: str
    DiscNumber: str
    Discord: str
    DiscordUrlInSlackNotification: str
    Discover: str
    DiskSpace: str
    Docker: str
    DockerUpdater: str
    Donations: str
    DoneEditingGroups: str
    DoNotPrefer: str
    DoNotUpgradeAutomatically: str
    Download: str
    DownloadClient: str
    DownloadClientCheckDownloadingToRoot: str
    DownloadClientCheckNoneAvailableMessage: str
    DownloadClientCheckUnableToCommunicateMessage: str
    DownloadClients: str
    DownloadClientSettings: str
    DownloadClientsSettingsSummary: str
    DownloadClientStatusCheckAllClientMessage: str
    DownloadClientStatusCheckSingleClientMessage: str
    DownloadClientUnavailable: str
    Downloaded: str
    DownloadedAndMonitored: str
    DownloadedButNotMonitored: str
    DownloadFailed: str
    DownloadFailedCheckDownloadClientForMoreDetails: str
    DownloadFailedInterp: str
    Downloading: str
    DownloadPropersAndRepacks: str
    DownloadPropersAndRepacksHelpText1: str
    DownloadPropersAndRepacksHelpText2: str
    DownloadPropersAndRepacksHelpTexts1: str
    DownloadPropersAndRepacksHelpTexts2: str
    DownloadPropersAndRepacksHelpTextWarning: str
    DownloadWarning: str
    DownloadWarningCheckDownloadClientForMoreDetails: str
    Edit: str
    EditAuthor: str
    EditCustomFormat: str
    EditDelayProfile: str
    EditGroups: str
    EditIndexer: str
    Edition: str
    EditionsHelpText: str
    EditListExclusion: str
    EditMovie: str
    EditMovieFile: str
    EditPerson: str
    EditQualityProfile: str
    EditRemotePathMapping: str
    EditRestriction: str
    EmbedMetadataHelpText: str
    EmbedMetadataInBookFiles: str
    Enable: str
    EnableAutoHelpText: str
    EnableAutomaticAdd: str
    EnableAutomaticAddHelpText: str
    EnableAutomaticSearch: str
    EnableAutomaticSearchHelpText: str
    EnableAutomaticSearchHelpTextWarning: str
    EnableColorImpairedMode: str
    EnableColorImpairedModeHelpText: str
    EnableCompletedDownloadHandlingHelpText: str
    Enabled: str
    EnabledHelpText: str
    EnableHelpText: str
    EnableInteractiveSearch: str
    EnableInteractiveSearchHelpText: str
    EnableInteractiveSearchHelpTextWarning: str
    EnableMediaInfoHelpText: str
    EnableProfile: str
    EnableRSS: str
    EnableSSL: str
    EnableSslHelpText: str
    Ended: str
    EndedAllBooksDownloaded: str
    EntityName: str
    Episode: str
    EpisodeDoesNotHaveAnAbsoluteEpisodeNumber: str
    Error: str
    ErrorLoadingContents: str
    ErrorLoadingPreviews: str
    ErrorRestoringBackup: str
    Events: str
    EventType: str
    Exception: str
    Excluded: str
    ExcludeMovie: str
    ExcludeTitle: str
    Existing: str
    ExistingBooks: str
    ExistingItems: str
    ExistingMovies: str
    ExistingTag: str
    ExistingTagsScrubbed: str
    ExportCustomFormat: str
    Extension: str
    ExternalUpdater: str
    ExtraFileExtensionsHelpTexts1: str
    ExtraFileExtensionsHelpTexts2: str
    Failed: str
    FailedDownloadHandling: str
    FailedLoadingSearchResults: str
    FailedToLoadMovieFromAPI: str
    FailedToLoadQueue: str
    FeatureRequests: str
    FileDateHelpText: str
    FileDetails: str
    FileManagement: str
    Filename: str
    FileNames: str
    FileNameTokens: str
    Files: str
    FilesTotal: str
    FileWasDeletedByUpgrade: str
    FileWasDeletedByViaUI: str
    Filter: str
    FilterAnalyticsEvents: str
    FilterAuthor: str
    FilterPlaceHolder: str
    Filters: str
    FilterSentryEventsHelpText: str
    FirstBook: str
    FirstDayOfWeek: str
    Fixed: str
    FocusSearchBox: str
    Folder: str
    FolderMoveRenameWarning: str
    Folders: str
    FollowPerson: str
    Forecast: str
    ForeignIdHelpText: str
    Formats: str
    ForMoreInformationOnTheIndividualDownloadClients: str
    ForMoreInformationOnTheIndividualDownloadClientsClickOnTheInfoButtons: str
    ForMoreInformationOnTheIndividualImportListsClinkOnTheInfoButtons: str
    ForMoreInformationOnTheIndividualIndexers: str
    ForMoreInformationOnTheIndividualIndexersClickOnTheInfoButtons: str
    ForMoreInformationOnTheIndividualListsClickOnTheInfoButtons: str
    FreeSpace: str
    From: str
    FutureBooks: str
    FutureDays: str
    FutureDaysHelpText: str
    General: str
    GeneralSettings: str
    GeneralSettingsSummary: str
    Genres: str
    Global: str
    GoToAuthorListing: str
    GoToInterp: str
    Grab: str
    Grabbed: str
    GrabID: str
    GrabRelease: str
    GrabReleaseMessageText: str
    GrabSelected: str
    Group: str
    HardlinkCopyFiles: str
    HasMonitoredBooksNoMonitoredBooksForThisAuthor: str
    HasPendingChangesNoChanges: str
    HasPendingChangesSaveChanges: str
    HaveNotAddedMovies: str
    Health: str
    HealthNoIssues: str
    HelpText: str
    HiddenClickToShow: str
    HideAdvanced: str
    HideBooks: str
    History: str
    HomePage: str
    Host: str
    HostHelpText: str
    Hostname: str
    Hours: str
    HttpHttps: str
    ICalFeed: str
    ICalHttpUrlHelpText: str
    iCalLink: str
    ICalLink: str
    IconForCutoffUnmet: str
    IconTooltip: str
    IfYouDontAddAnImportListExclusionAndTheAuthorHasAMetadataProfileOtherThanNoneThenThisBookMayBeReaddedDuringTheNextAuthorRefresh: str
    Ignored: str
    IgnoredAddresses: str
    IgnoreDeletedBooks: str
    IgnoreDeletedMovies: str
    IgnoredHelpText: str
    IgnoredMetaHelpText: str
    IgnoredPlaceHolder: str
    IllRestartLater: str
    Images: str
    IMDb: str
    ImdbRating: str
    ImdbVotes: str
    Import: str
    ImportCustomFormat: str
    Imported: str
    ImportedTo: str
    ImportErrors: str
    ImportExistingMovies: str
    ImportExtraFiles: str
    ImportExtraFilesHelpText: str
    ImportFailed: str
    ImportFailedInterp: str
    ImportFailures: str
    ImportHeader: str
    ImportIncludeQuality: str
    Importing: str
    ImportLibrary: str
    ImportListExclusions: str
    ImportListMissingRoot: str
    ImportListMultipleMissingRoots: str
    ImportLists: str
    ImportListSettings: str
    ImportListSpecificSettings: str
    ImportListStatusCheckAllClientMessage: str
    ImportListStatusCheckSingleClientMessage: str
    ImportListSyncIntervalHelpText: str
    ImportMechanismHealthCheckMessage: str
    ImportMovies: str
    ImportNotForDownloads: str
    ImportRootPath: str
    ImportTipsMessage: str
    InCinemas: str
    InCinemasDate: str
    InCinemasMsg: str
    IncludeCustomFormatWhenRenaming: str
    IncludeCustomFormatWhenRenamingHelpText: str
    IncludeHealthWarningsHelpText: str
    IncludePreferredWhenRenaming: str
    IncludeRadarrRecommendations: str
    IncludeRecommendationsHelpText: str
    IncludeUnknownAuthorItemsHelpText: str
    IncludeUnknownMovieItemsHelpText: str
    IncludeUnmonitored: str
    Indexer: str
    IndexerDownloadClientHelpText: str
    IndexerFlags: str
    IndexerIdHelpText: str
    IndexerIdHelpTextWarning: str
    IndexerIdvalue0IncludeInPreferredWordsRenamingFormat: str
    IndexerIdvalue0OnlySupportedWhenIndexerIsSetToAll: str
    IndexerJackettAll: str
    IndexerLongTermStatusCheckAllClientMessage: str
    IndexerLongTermStatusCheckSingleClientMessage: str
    IndexerPriority: str
    IndexerPriorityHelpText: str
    IndexerRssHealthCheckNoAvailableIndexers: str
    IndexerRssHealthCheckNoIndexers: str
    Indexers: str
    IndexerSearchCheckNoAutomaticMessage: str
    IndexerSearchCheckNoAvailableIndexersMessage: str
    IndexerSearchCheckNoInteractiveMessage: str
    IndexerSettings: str
    IndexersSettingsSummary: str
    IndexerStatusCheckAllClientMessage: str
    IndexerStatusCheckSingleClientMessage: str
    IndexerTagHelpText: str
    Info: str
    InstallLatest: str
    InteractiveImport: str
    InteractiveImportErrLanguage: str
    InteractiveImportErrMovie: str
    InteractiveImportErrQuality: str
    InteractiveSearch: str
    Interval: str
    InvalidFormat: str
    ISBN: str
    IsCalibreLibraryHelpText: str
    IsCutoffCutoff: str
    IsCutoffUpgradeUntilThisQualityIsMetOrExceeded: str
    IsExpandedHideBooks: str
    IsExpandedHideFileInfo: str
    IsExpandedShowBooks: str
    IsExpandedShowFileInfo: str
    IsInUseCantDeleteAMetadataProfileThatIsAttachedToAnAuthorOrImportList: str
    IsInUseCantDeleteAQualityProfileThatIsAttachedToAnAuthorOrImportList: str
    IsShowingMonitoredMonitorSelected: str
    IsShowingMonitoredUnmonitorSelected: str
    IsTagUsedCannotBeDeletedWhileInUse: str
    KeepAndUnmonitorMovie: str
    KeyboardShortcuts: str
    Label: str
    Language: str
    LanguageHelpText: str
    Languages: str
    Large: str
    LastDuration: str
    LastExecution: str
    LastUsed: str
    LastWriteTime: str
    LatestBook: str
    LaunchBrowserHelpText: str
    Letterboxd: str
    Level: str
    LibraryHelpText: str
    LinkHere: str
    Links: str
    ListExclusions: str
    Lists: str
    ListSettings: str
    ListsSettingsSummary: str
    ListSyncLevelHelpText: str
    ListSyncLevelHelpTextWarning: str
    ListTagsHelpText: str
    ListUpdateInterval: str
    LoadingBookFilesFailed: str
    LoadingBooksFailed: str
    LoadingMovieCreditsFailed: str
    LoadingMovieExtraFilesFailed: str
    LoadingMovieFilesFailed: str
    Local: str
    LocalPath: str
    LocalPathHelpText: str
    Location: str
    LogFiles: str
    Logging: str
    LogLevel: str
    LogLevelTraceHelpTextWarning: str
    LogLevelvalueTraceTraceLoggingShouldOnlyBeEnabledTemporarily: str
    LogOnly: str
    LogRotateHelpText: str
    LogRotation: str
    Logs: str
    LogSQL: str
    LogSqlHelpText: str
    LongDateFormat: str
    LookingForReleaseProfiles1: str
    LookingForReleaseProfiles2: str
    LowerCase: str
    MaintenanceRelease: str
    Manual: str
    ManualDownload: str
    ManualImport: str
    ManualImportSelectLanguage: str
    ManualImportSelectMovie: str
    ManualImportSelectQuality: str
    ManualImportSetReleaseGroup: str
    MappedDrivesRunningAsService: str
    MarkAsFailed: str
    MarkAsFailedMessageText: str
    MassBookSearch: str
    MassBookSearchWarning: str
    MassMovieSearch: str
    Max: str
    MaximumLimits: str
    MaximumSize: str
    MaximumSizeHelpText: str
    Mechanism: str
    MediaInfo: str
    MediaManagement: str
    MediaManagementSettings: str
    MediaManagementSettingsSummary: str
    Medium: str
    MediumFormat: str
    MegabytesPerMinute: str
    Message: str
    Metadata: str
    MetadataConsumers: str
    MetadataProfile: str
    MetadataProfileIdHelpText: str
    MetadataProfiles: str
    MetadataProviderSource: str
    MetadataSettings: str
    MetadataSettingsSummary: str
    MetadataSource: str
    MetadataSourceHelpText: str
    MIA: str
    Min: str
    MinAvailability: str
    MinFormatScoreHelpText: str
    MinimumAge: str
    MinimumAgeHelpText: str
    MinimumAvailability: str
    MinimumCustomFormatScore: str
    MinimumFreeSpace: str
    MinimumFreeSpaceWhenImportingHelpText: str
    MinimumLimits: str
    MinimumPages: str
    MinimumPopularity: str
    MinPagesHelpText: str
    MinPopularityHelpText: str
    Minutes: str
    MinutesHundredTwenty: str
    MinutesNinety: str
    MinutesSixty: str
    Missing: str
    MissingBooks: str
    MissingBooksAuthorMonitored: str
    MissingBooksAuthorNotMonitored: str
    MissingFromDisk: str
    MissingMonitoredAndConsideredAvailable: str
    MissingNotMonitored: str
    Mode: str
    Monday: str
    Monitor: str
    MonitorAuthor: str
    MonitorBook: str
    MonitorBookExistingOnlyWarning: str
    Monitored: str
    MonitoredAuthorIsMonitored: str
    MonitoredAuthorIsUnmonitored: str
    MonitoredHelpText: str
    MonitoredOnly: str
    MonitoredStatus: str
    Monitoring: str
    MonitoringOptions: str
    MonitoringOptionsHelpText: str
    MonitorMovie: str
    MonitorNewItems: str
    MonitorNewItemsHelpText: str
    MonoVersion: str
    Month: str
    Months: str
    More: str
    MoreControlCFText: str
    MoreDetails: str
    MoreInfo: str
    MountCheckMessage: str
    MoveFiles: str
    MoveFolders1: str
    MoveFolders2: str
    Movie: str
    MovieAlreadyExcluded: str
    MovieChat: str
    MovieDetailsNextMovie: str
    MovieDetailsPreviousMovie: str
    MovieEditor: str
    MovieExcludedFromAutomaticAdd: str
    MovieFiles: str
    MovieFilesTotaling: str
    MovieFolderFormat: str
    MovieID: str
    MovieIndex: str
    MovieIndexScrollBottom: str
    MovieIndexScrollTop: str
    MovieInfoLanguage: str
    MovieInfoLanguageHelpText: str
    MovieInfoLanguageHelpTextWarning: str
    MovieInvalidFormat: str
    MovieIsDownloading: str
    MovieIsDownloadingInterp: str
    MovieIsMonitored: str
    MovieIsOnImportExclusionList: str
    MovieIsRecommend: str
    MovieIsUnmonitored: str
    MovieNaming: str
    Movies: str
    MoviesSelectedInterp: str
    MovieTitle: str
    MovieTitleHelpText: str
    MovieYear: str
    MovieYearHelpText: str
    MultiLanguage: str
    MusicBrainzAuthorID: str
    MusicBrainzBookID: str
    MusicbrainzId: str
    MusicBrainzRecordingID: str
    MusicBrainzReleaseID: str
    MusicBrainzTrackID: str
    MustContain: str
    MustNotContain: str
    Name: str
    NameFirstLast: str
    NameLastFirst: str
    NameStyle: str
    NamingSettings: str
    Negate: str
    Negated: str
    NegateHelpText: str
    NetCore: str
    NETCore: str
    New: str
    NewBooks: str
    NextExecution: str
    No: str
    NoAltTitle: str
    NoBackupsAreAvailable: str
    NoChange: str
    NoChanges: str
    NoEventsFound: str
    NoHistory: str
    NoHistoryBlocklist: str
    NoLeaveIt: str
    NoLimitForAnyRuntime: str
    NoLinks: str
    NoListRecommendations: str
    NoLogFiles: str
    NoMatchFound: str
    NoMinimumForAnyRuntime: str
    NoMoveFilesSelf: str
    NoMoviesExist: str
    NoName: str
    NoResultsFound: str
    NoTagsHaveBeenAddedYet: str
    NotAvailable: str
    NotificationTriggers: str
    NotificationTriggersHelpText: str
    NotMonitored: str
    NoUpdatesAreAvailable: str
    NoVideoFilesFoundSelectedFolder: str
    OAuthPopupMessage: str
    Ok: str
    OnApplicationUpdate: str
    OnApplicationUpdateHelpText: str
    OnBookRetagHelpText: str
    OnDownloadFailureHelpText: str
    OnDownloadHelpText: str
    OnGrab: str
    OnGrabHelpText: str
    OnHealthIssue: str
    OnHealthIssueHelpText: str
    OnImport: str
    OnImportFailureHelpText: str
    OnLatestVersion: str
    OnlyTorrent: str
    OnlyUsenet: str
    OnMovieDelete: str
    OnMovieDeleteHelpText: str
    OnMovieFileDelete: str
    OnMovieFileDeleteForUpgrade: str
    OnMovieFileDeleteForUpgradeHelpText: str
    OnMovieFileDeleteHelpText: str
    OnReleaseImportHelpText: str
    OnRename: str
    OnRenameHelpText: str
    OnUpgrade: str
    OnUpgradeHelpText: str
    OpenBrowserOnStart: str
    OpenThisModal: str
    Options: str
    Organize: str
    OrganizeAndRename: str
    OrganizeConfirm: str
    OrganizeModalAllPathsRelative: str
    OrganizeModalDisabled: str
    OrganizeModalNamingPattern: str
    OrganizeModalSuccess: str
    OrganizeSelectedMovies: str
    Original: str
    Other: str
    OutputFormatHelpText: str
    OutputPath: str
    Overview: str
    OverviewOptions: str
    PackageVersion: str
    PageSize: str
    PageSizeHelpText: str
    Password: str
    PasswordHelpText: str
    PastDays: str
    PastDaysHelpText: str
    Path: str
    PathHelpText: str
    PathHelpTextWarning: str
    Paused: str
    Peers: str
    Pending: str
    PendingChangesDiscardChanges: str
    PendingChangesMessage: str
    PendingChangesStayReview: str
    Permissions: str
    PhysicalRelease: str
    PhysicalReleaseDate: str
    Port: str
    PortHelpText: str
    PortHelpTextWarning: str
    PortNumber: str
    PosterOptions: str
    Posters: str
    PosterSize: str
    PreferAndUpgrade: str
    PreferIndexerFlags: str
    PreferIndexerFlagsHelpText: str
    Preferred: str
    PreferredHelpTexts1: str
    PreferredHelpTexts2: str
    PreferredHelpTexts3: str
    PreferredSize: str
    PreferTorrent: str
    PreferUsenet: str
    Presets: str
    PreviewRename: str
    PreviewRenameHelpText: str
    PreviewRetag: str
    Priority: str
    PriorityHelpText: str
    PrioritySettings: str
    ProcessingFolders: str
    Profiles: str
    ProfilesSettingsSummary: str
    Progress: str
    Proper: str
    PropersAndRepacks: str
    Protocol: str
    ProtocolHelpText: str
    Proxy: str
    ProxyBypassFilterHelpText: str
    ProxyCheckBadRequestMessage: str
    ProxyCheckFailedToTestMessage: str
    ProxyCheckResolveIpMessage: str
    ProxyPasswordHelpText: str
    ProxyType: str
    ProxyUsernameHelpText: str
    PtpOldSettingsCheckMessage: str
    PublishedDate: str
    Publisher: str
    Qualities: str
    QualitiesHelpText: str
    Quality: str
    QualityCutoffHasNotBeenMet: str
    QualityDefinitions: str
    QualityLimitsHelpText: str
    QualityOrLangCutoffHasNotBeenMet: str
    QualityProfile: str
    QualityProfileDeleteConfirm: str
    QualityProfileIdHelpText: str
    QualityProfileInUse: str
    QualityProfiles: str
    QualitySettings: str
    QualitySettingsSummary: str
    Queue: str
    Queued: str
    QueueIsEmpty: str
    QuickImport: str
    RadarrCalendarFeed: str
    RadarrSupportsAnyDownloadClient: str
    RadarrSupportsAnyIndexer: str
    RadarrSupportsAnyRSSMovieListsAsWellAsTheOneStatedBelow: str
    RadarrSupportsCustomConditionsAgainstTheReleasePropertiesBelow: str
    RadarrTags: str
    RadarrUpdated: str
    Ratings: str
    ReadarrSupportsAnyDownloadClientThatUsesTheNewznabStandardAsWellAsOtherDownloadClientsListedBelow: str
    ReadarrSupportsAnyIndexerThatUsesTheNewznabStandardAsWellAsOtherIndexersListedBelow: str
    ReadarrSupportsMultipleListsForImportingBooksAndAuthorsIntoTheDatabase: str
    ReadarrTags: str
    ReadTheWikiForMoreInformation: str
    Real: str
    Reason: str
    RecentChanges: str
    RecentFolders: str
    RecycleBinCleanupDaysHelpText: str
    RecycleBinCleanupDaysHelpTextWarning: str
    RecycleBinHelpText: str
    RecyclingBin: str
    RecyclingBinCleanup: str
    Reddit: str
    Redownload: str
    Refresh: str
    RefreshAndScan: str
    RefreshAuthor: str
    RefreshInformation: str
    RefreshInformationAndScanDisk: str
    RefreshLists: str
    RefreshMovie: str
    RefreshScan: str
    RegularExpressionsCanBeTested: str
    RejectionCount: str
    RelativePath: str
    ReleaseBranchCheckOfficialBranchMessage: str
    Released: str
    ReleaseDate: str
    ReleaseDates: str
    ReleasedMsg: str
    ReleaseGroup: str
    ReleaseProfiles: str
    ReleaseRejected: str
    ReleaseStatus: str
    ReleaseTitle: str
    ReleaseWillBeProcessedInterp: str
    Reload: str
    RemotePath: str
    RemotePathHelpText: str
    RemotePathMappingCheckBadDockerPath: str
    RemotePathMappingCheckDockerFolderMissing: str
    RemotePathMappingCheckDownloadPermissions: str
    RemotePathMappingCheckFileRemoved: str
    RemotePathMappingCheckFilesBadDockerPath: str
    RemotePathMappingCheckFilesGenericPermissions: str
    RemotePathMappingCheckFilesLocalWrongOSPath: str
    RemotePathMappingCheckFilesWrongOSPath: str
    RemotePathMappingCheckFolderPermissions: str
    RemotePathMappingCheckGenericPermissions: str
    RemotePathMappingCheckImportFailed: str
    RemotePathMappingCheckLocalFolderMissing: str
    RemotePathMappingCheckLocalWrongOSPath: str
    RemotePathMappingCheckRemoteDownloadClient: str
    RemotePathMappingCheckWrongOSPath: str
    RemotePathMappings: str
    Remove: str
    RemoveCompleted: str
    RemoveCompletedDownloadsHelpText: str
    RemovedFromTaskQueue: str
    RemovedMovieCheckMultipleMessage: str
    RemovedMovieCheckSingleMessage: str
    RemoveDownloadsAlert: str
    RemoveFailed: str
    RemoveFailedDownloadsHelpText: str
    RemoveFilter: str
    RemoveFromBlocklist: str
    RemoveFromDownloadClient: str
    RemoveFromQueue: str
    RemoveFromQueueText: str
    RemoveHelpTextWarning: str
    RemoveMovieAndDeleteFiles: str
    RemoveMovieAndKeepFiles: str
    RemoveRootFolder: str
    RemoveSelected: str
    RemoveSelectedItem: str
    RemoveSelectedItems: str
    RemoveSelectedMessageText: str
    RemoveTagExistingTag: str
    RemoveTagRemovingTag: str
    RemovingTag: str
    RenameBooks: str
    RenameBooksHelpText: str
    Renamed: str
    RenameFiles: str
    RenameMovies: str
    RenameMoviesHelpText: str
    Reorder: str
    Replace: str
    ReplaceIllegalCharacters: str
    ReplaceIllegalCharactersHelpText: str
    ReplaceWithDash: str
    ReplaceWithSpaceDash: str
    ReplaceWithSpaceDashSpace: str
    Required: str
    RequiredHelpText: str
    RequiredPlaceHolder: str
    RequiredRestrictionHelpText: str
    RequiredRestrictionPlaceHolder: str
    RescanAfterRefreshHelpText: str
    RescanAfterRefreshHelpTextWarning: str
    RescanAuthorFolderAfterRefresh: str
    RescanMovieFolderAfterRefresh: str
    Reset: str
    ResetAPIKey: str
    ResetAPIKeyMessageText: str
    Restart: str
    RestartNow: str
    RestartRadarr: str
    RestartReadarr: str
    RestartReloadNote: str
    RestartRequiredHelpTextWarning: str
    Restore: str
    RestoreBackup: str
    Restrictions: str
    Result: str
    Retention: str
    RetentionHelpText: str
    RetryingDownloadInterp: str
    RootFolder: str
    RootFolderCheckMultipleMessage: str
    RootFolderCheckSingleMessage: str
    RootFolderPathHelpText: str
    RootFolders: str
    RSS: str
    RSSIsNotSupportedWithThisIndexer: str
    RSSSync: str
    RSSSyncInterval: str
    RssSyncIntervalHelpText: str
    RSSSyncIntervalHelpTextWarning: str
    Runtime: str
    Save: str
    SaveChanges: str
    SaveSettings: str
    SceneInformation: str
    SceneNumberHasntBeenVerifiedYet: str
    Scheduled: str
    Score: str
    Script: str
    ScriptPath: str
    Search: str
    SearchAll: str
    SearchBook: str
    SearchBoxPlaceHolder: str
    SearchCutoffUnmet: str
    SearchFailedPleaseTryAgainLater: str
    SearchFiltered: str
    SearchForAllCutoffUnmetBooks: str
    SearchForAllMissingBooks: str
    SearchForMissing: str
    SearchForMonitoredBooks: str
    SearchForMovie: str
    SearchForNewItems: str
    SearchMissing: str
    SearchMonitored: str
    SearchMovie: str
    SearchOnAdd: str
    SearchOnAddHelpText: str
    SearchSelected: str
    Season: str
    Seconds: str
    Security: str
    Seeders: str
    SelectAll: str
    SelectDotDot: str
    SelectedCountAuthorsSelectedInterp: str
    SelectedCountBooksSelectedInterp: str
    SelectFolder: str
    SelectLanguage: str
    SelectLanguages: str
    SelectMovie: str
    SelectQuality: str
    SelectReleaseGroup: str
    SendAnonymousUsageData: str
    SendMetadataToCalibre: str
    Series: str
    SeriesNumber: str
    SeriesTotal: str
    SetPermissions: str
    SetPermissionsLinuxHelpText: str
    SetPermissionsLinuxHelpTextWarning: str
    SetReleaseGroup: str
    SetTags: str
    Settings: str
    SettingsEnableColorImpairedMode: str
    SettingsEnableColorImpairedModeHelpText: str
    SettingsFirstDayOfWeek: str
    SettingsLongDateFormat: str
    SettingsRemotePathMappingHostHelpText: str
    SettingsRemotePathMappingLocalPath: str
    SettingsRemotePathMappingLocalPathHelpText: str
    SettingsRemotePathMappingRemotePath: str
    SettingsRemotePathMappingRemotePathHelpText: str
    SettingsRuntimeFormat: str
    SettingsShortDateFormat: str
    SettingsShowRelativeDates: str
    SettingsShowRelativeDatesHelpText: str
    SettingsTimeFormat: str
    SettingsWeekColumnHeader: str
    SettingsWeekColumnHeaderHelpText: str
    ShortDateFormat: str
    ShouldMonitorExisting: str
    ShouldMonitorExistingHelpText: str
    ShouldMonitorHelpText: str
    ShouldSearchHelpText: str
    ShowAdvanced: str
    ShowAsAllDayEvents: str
    ShowBanners: str
    ShowBannersHelpText: str
    ShowBookCount: str
    ShowBookTitleHelpText: str
    ShowCertification: str
    ShowCinemaRelease: str
    showCinemaReleaseHelpText: str
    ShowCutoffUnmetIconHelpText: str
    ShowDateAdded: str
    ShowGenres: str
    ShowLastBook: str
    ShowMonitored: str
    ShowMonitoredHelpText: str
    ShowMovieInformation: str
    ShowMovieInformationHelpText: str
    ShownAboveEachColumnWhenWeekIsTheActiveView: str
    ShowName: str
    ShownClickToHide: str
    ShowPath: str
    ShowQualityProfile: str
    ShowQualityProfileHelpText: str
    ShowRatings: str
    ShowRelativeDates: str
    ShowRelativeDatesHelpText: str
    ShowReleaseDate: str
    ShowReleaseDateHelpText: str
    ShowSearch: str
    ShowSearchActionHelpText: str
    ShowSearchHelpText: str
    ShowSizeOnDisk: str
    ShowStudio: str
    ShowTitle: str
    ShowTitleHelpText: str
    ShowUnknownAuthorItems: str
    ShowUnknownMovieItems: str
    ShowYear: str
    Shutdown: str
    Size: str
    SizeLimit: str
    SizeOnDisk: str
    SkipBooksWithMissingReleaseDate: str
    SkipBooksWithNoISBNOrASIN: str
    SkipFreeSpaceCheck: str
    SkipFreeSpaceCheckWhenImportingHelpText: str
    SkipPartBooksAndSets: str
    SkipRedownload: str
    SkipredownloadHelpText: str
    SkipSecondarySeriesBooks: str
    Small: str
    Socks4: str
    Socks5: str
    SomeResultsHiddenFilter: str
    SorryThatAuthorCannotBeFound: str
    SorryThatBookCannotBeFound: str
    SorryThatMovieCannotBeFound: str
    Sort: str
    Source: str
    SourcePath: str
    SourceRelativePath: str
    SourceTitle: str
    SpecificBook: str
    SqliteVersionCheckUpgradeRequiredMessage: str
    SSLCertPassword: str
    SslCertPasswordHelpText: str
    SSLCertPasswordHelpText: str
    SslCertPasswordHelpTextWarning: str
    SSLCertPath: str
    SslCertPathHelpText: str
    SSLCertPathHelpText: str
    SslCertPathHelpTextWarning: str
    SSLPort: str
    SslPortHelpTextWarning: str
    StandardBookFormat: str
    StandardMovieFormat: str
    StartImport: str
    StartProcessing: str
    StartSearchForMissingMovie: str
    StartTypingOrSelectAPathBelow: str
    StartupDirectory: str
    Status: str
    StatusEndedContinuing: str
    StatusEndedDeceased: str
    StatusEndedEnded: str
    Studio: str
    Style: str
    SubfolderWillBeCreatedAutomaticallyInterp: str
    SuccessMyWorkIsDoneNoFilesToRename: str
    SuccessMyWorkIsDoneNoFilesToRetag: str
    SuggestTranslationChange: str
    Sunday: str
    SupportsRssvalueRSSIsNotSupportedWithThisIndexer: str
    SupportsSearchvalueSearchIsNotSupportedWithThisIndexer: str
    SupportsSearchvalueWillBeUsedWhenAutomaticSearchesArePerformedViaTheUIOrByReadarr: str
    SupportsSearchvalueWillBeUsedWhenInteractiveSearchIsUsed: str
    System: str
    SystemTimeCheckMessage: str
    Table: str
    TableOptions: str
    TableOptionsColumnsMessage: str
    TagCannotBeDeletedWhileInUse: str
    TagDetails: str
    TagIsNotUsedAndCanBeDeleted: str
    Tags: str
    TagsHelpText: str
    TagsSettingsSummary: str
    Tasks: str
    TaskUserAgentTooltip: str
    TBA: str
    Term: str
    Test: str
    TestAll: str
    TestAllClients: str
    TestAllIndexers: str
    TestAllLists: str
    TheAuthorFolderAndAllOfItsContentWillBeDeleted: str
    TheBooksFilesWillBeDeleted: str
    TheFollowingFilesWillBeDeleted: str
    TheLogLevelDefault: str
    ThisCannotBeCancelled: str
    ThisConditionMatchesUsingRegularExpressions: str
    ThisWillApplyToAllIndexersPleaseFollowTheRulesSetForthByThem: str
    Time: str
    TimeFormat: str
    Timeleft: str
    Title: str
    Titles: str
    TMDb: str
    TMDBId: str
    TmdbIdHelpText: str
    TmdbRating: str
    TmdbVotes: str
    Today: str
    Tomorrow: str
    TooManyBooks: str
    TorrentDelay: str
    TorrentDelayHelpText: str
    TorrentDelayTime: str
    Torrents: str
    TorrentsDisabled: str
    TotalBookCountBooksTotalBookFileCountBooksWithFilesInterp: str
    TotalFileSize: str
    TotalSpace: str
    Trace: str
    TrackNumber: str
    TrackTitle: str
    Trailer: str
    Trakt: str
    Trigger: str
    Type: str
    UI: str
    UILanguage: str
    UILanguageHelpText: str
    UILanguageHelpTextWarning: str
    UISettings: str
    UISettingsSummary: str
    UnableToAddANewConditionPleaseTryAgain: str
    UnableToAddANewCustomFormatPleaseTryAgain: str
    UnableToAddANewDownloadClientPleaseTryAgain: str
    UnableToAddANewImportListExclusionPleaseTryAgain: str
    UnableToAddANewIndexerPleaseTryAgain: str
    UnableToAddANewListExclusionPleaseTryAgain: str
    UnableToAddANewListPleaseTryAgain: str
    UnableToAddANewMetadataProfilePleaseTryAgain: str
    UnableToAddANewNotificationPleaseTryAgain: str
    UnableToAddANewQualityProfilePleaseTryAgain: str
    UnableToAddANewRemotePathMappingPleaseTryAgain: str
    UnableToAddANewRootFolderPleaseTryAgain: str
    UnableToAddRootFolder: str
    UnableToImportCheckLogs: str
    UnableToLoadAltTitle: str
    UnableToLoadBackups: str
    UnableToLoadBlocklist: str
    UnableToLoadCustomFormats: str
    UnableToLoadDelayProfiles: str
    UnableToLoadDownloadClientOptions: str
    UnableToLoadDownloadClients: str
    UnableToLoadGeneralSettings: str
    UnableToLoadHistory: str
    UnableToLoadImportListExclusions: str
    UnableToLoadIndexerOptions: str
    UnableToLoadIndexers: str
    UnableToLoadLanguages: str
    UnableToLoadListExclusions: str
    UnableToLoadListOptions: str
    UnableToLoadLists: str
    UnableToLoadManualImportItems: str
    UnableToLoadMediaManagementSettings: str
    UnableToLoadMetadata: str
    UnableToLoadMetadataProfiles: str
    UnableToLoadMetadataProviderSettings: str
    UnableToLoadMovies: str
    UnableToLoadNamingSettings: str
    UnableToLoadNotifications: str
    UnableToLoadQualities: str
    UnableToLoadQualityDefinitions: str
    UnableToLoadQualityProfiles: str
    UnableToLoadReleaseProfiles: str
    UnableToLoadRemotePathMappings: str
    UnableToLoadRestrictions: str
    UnableToLoadResultsIntSearch: str
    UnableToLoadRootFolders: str
    UnableToLoadTags: str
    UnableToLoadTheCalendar: str
    UnableToLoadUISettings: str
    UnableToUpdateRadarrDirectly: str
    Unavailable: str
    Ungroup: str
    Unlimited: str
    UnmappedFiles: str
    UnmappedFilesOnly: str
    UnmappedFolders: str
    Unmonitored: str
    UnmonitoredHelpText: str
    Unreleased: str
    UnsavedChanges: str
    UnselectAll: str
    UpdateAll: str
    UpdateAutomaticallyHelpText: str
    UpdateAvailable: str
    UpdateCheckStartupNotWritableMessage: str
    UpdateCheckStartupTranslocationMessage: str
    UpdateCheckUINotWritableMessage: str
    UpdateCovers: str
    UpdateCoversHelpText: str
    UpdateMechanismHelpText: str
    Updates: str
    UpdateScriptPathHelpText: str
    UpdateSelected: str
    UpdatingIsDisabledInsideADockerContainerUpdateTheContainerImageInstead: str
    UpgradeAllowedHelpText: str
    UpgradesAllowed: str
    UpgradeUntilCustomFormatScore: str
    UpgradeUntilQuality: str
    UpgradeUntilThisQualityIsMetOrExceeded: str
    UpperCase: str
    Uptime: str
    URLBase: str
    UrlBaseHelpText: str
    UrlBaseHelpTextWarning: str
    UseCalibreContentServer: str
    UseHardlinksInsteadOfCopy: str
    Usenet: str
    UsenetDelay: str
    UsenetDelayHelpText: str
    UsenetDelayTime: str
    UsenetDisabled: str
    UseProxy: str
    Username: str
    UsernameHelpText: str
    UseSSL: str
    UseSslHelpText: str
    UsingExternalUpdateMechanismBranchToUseToUpdateReadarr: str
    UsingExternalUpdateMechanismBranchUsedByExternalUpdateMechanism: str
    Version: str
    VersionUpdateText: str
    VideoCodec: str
    View: str
    VisitGithubCustomFormatsAphrodite: str
    WaitingToImport: str
    WaitingToProcess: str
    Wanted: str
    Warn: str
    WatchLibraryForChangesHelpText: str
    WatchRootFoldersForFileChanges: str
    Week: str
    WeekColumnHeader: str
    Weeks: str
    WhatsNew: str
    WhitelistedHardcodedSubsHelpText: str
    WhitelistedSubtitleTags: str
    Wiki: str
    WouldYouLikeToRestoreBackup: str
    WriteAudioTags: str
    WriteAudioTagsScrub: str
    WriteAudioTagsScrubHelp: str
    WriteBookTagsHelpTextWarning: str
    WriteTagsAll: str
    WriteTagsNew: str
    WriteTagsNo: str
    WriteTagsSync: str
    Year: str
    Yes: str
    YesCancel: str
    YesMoveFiles: str
    Yesterday: str
    YouCanAlsoSearch: str


@dataclass(init=False)
class _LogRecord(BaseModel):
    """Log record attributes."""

    exception: str
    exceptionType: str
    id: int
    level: str
    logger: str
    message: str
    time: datetime


@dataclass(init=False)
class _QualityProfileItems(_Common3):
    """Quality profile items attributes."""

    allowed: bool
    items: list[_QualityProfileItems] | None = None
    quality: type[_QualityInfo] = field(default=_QualityInfo)

    def __post_init__(self):
        self.items = [_QualityProfileItems(item) for item in self.items or []]
        self.quality = _QualityInfo(self.quality)


@dataclass(init=False)
class _ReleaseCommon(BaseModel):
    """Release common attributes."""

    age: int
    ageHours: float
    ageMinutes: float
    approved: bool
    commentUrl: str
    downloadAllowed: bool
    downloadUrl: str
    guid: str
    indexer: str
    indexerId: int
    infoHash: str
    infoUrl: str
    leechers: int
    magnetUrl: str
    protocol: ProtocolType
    publishDate: datetime
    qualityWeight: int
    rejected: bool
    rejections: list[_Rejection] | None = None
    releaseWeight: int
    sceneSource: bool
    seeders: int
    size: int
    temporarilyRejected: bool
    title: str

    def __post_init__(self):
        """Post init."""
        self.rejections = [_Rejection(x) for x in self.rejections or []]


@dataclass(init=False)
class _RecordCommon(BaseModel):
    """Record common attributes."""

    page: int
    pageSize: int
    sortDirection: str
    sortKey: str
    totalRecords: int


@dataclass(init=False)
class _ReleaseProfilePreferred(BaseModel):
    """Release profile preferred attributes."""

    key: str
    value: int


@dataclass(init=False)
class _Rename(BaseModel):
    """Rename attributes."""

    existingPath: str
    newPath: str


@dataclass(init=False)
class _Tag(BaseModel):
    """Tag attributes."""

    id: int
    label: str


@dataclass(init=False)
class _TagDetails(_Tag):
    """Tag details attributes."""

    delayProfileIds: list[int]
    importListIds: list[int]
    notificationIds: list[int]
    restrictionIds: list[int]


@dataclass(init=False)
class _UpdateChanges(BaseModel):
    """Update changes attributes."""

    fixed: list[str]
    new: list[str]


@dataclass(init=False)
class _TitleInfo(BaseModel):
    """Title info attributes."""

    title: str
    titleWithoutYear: str
    year: int


@dataclass(init=False)
class _Notification(BaseModel):
    """Notification attributes."""

    configContract: str
    implementation: str
    implementationName: str
    includeHealthWarnings: bool
    infoLink: str
    onDownload: bool
    onGrab: bool
    onHealthIssue: bool
    onRename: bool
    onUpgrade: bool
    supportsOnDownload: bool
    supportsOnGrab: bool
    supportsOnHealthIssue: bool
    supportsOnRename: bool
    supportsOnUpgrade: bool
    tags: list[int]


@dataclass(init=False)
class _RetagChange(BaseModel):
    """Retag change attributes."""

    field: str
    oldValue: str
    newValue: str


@dataclass(init=False)
class _HistoryCommon(BaseModel):
    """History common attributes."""

    age: int
    ageHours: float
    ageMinutes: float
    downloadClientName: str
    downloadUrl: str
    droppedPath: str
    fileId: int
    importedPath: str
    indexer: str
    nzbInfoUrl: str
    protocol: ProtocolType
    publishedDate: datetime
    reason: str
    releaseGroup: str
    size: int
    torrentInfoHash: str


@dataclass(init=False)
class _HistoryData(_HistoryCommon):
    """History data attributes."""

    downloadClient: str
    downloadForced: bool
    guid: str


@dataclass(init=False)
class _QualityCommon(BaseModel):
    """Quality common attributes."""

    quality: type[_Quality] = field(default=_Quality)
    qualityCutoffNotMet: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _Ratings(BaseModel):
    """Ratings attributes."""

    value: float
    votes: int


@dataclass(init=False)
class _Link(BaseModel):
    """Link attributes."""

    name: str
    url: str


@dataclass(init=False)
class _StatusMessage(BaseModel):
    """Status message attributes."""

    messages: list[str]
    title: str


@dataclass(init=False)
class _Editor(BaseModel):
    """Editor attributes."""

    applyTags: str
    deleteFiles: bool
    minimumAvailability: str
    monitored: bool
    moveFiles: bool
    qualityProfileId: int
    rootFolderPath: str
    tags: list[int]


@dataclass(init=False)
class _IsLoaded(BaseModel):
    """Is loaded attribute."""

    isLoaded: bool


@dataclass(init=False)
class _Rejection(BaseModel):
    """Rejection attributes."""

    reason: str
    type: str


@dataclass(init=False)
class _ManualImport(BaseModel):
    """Manual import attributes."""

    downloadId: str
    id: int
    name: str
    path: str
    quality: type[_Quality] = field(default=_Quality)
    qualityWeight: int
    rejections: list[_Rejection] | None = None
    size: int

    def __post_init__(self):
        """Post init."""
        self.quality = _Quality(self.quality)
        self.rejections = [_Rejection(x) for x in self.rejections or []]


@dataclass(init=False)
class _Monitor(BaseModel):
    """Sonarr series monitor attributes."""

    id: int
    monitored: bool


@dataclass(init=False)
class _MonitorOption(BaseModel):
    """Sonarr series monitor option attributes."""

    monitor: str | None = None



"""An asynchronous client for Lidarr/Radarr/Readarr/Sonarr APIs."""

from .exceptions import *  # noqa: F401, F403
from .models.lidarr import *  # noqa: F401, F403
from .models.radarr import *  # noqa: F401, F403
from .models.readarr import *  # noqa: F401, F403
from .models.request import *  # noqa: F401, F403
from .models.sonarr import *  # noqa: F401, F403



"""PyArrHostConfiguration."""
from __future__ import annotations

from dataclasses import dataclass

from aiopyarr.const import HEADERS

from .. import ArrException


@dataclass
class PyArrHostConfiguration:  # pylint: disable=too-many-instance-attributes
    """PyArrHostConfiguration."""

    api_token: str | None = None
    hostname: str | None = None
    ipaddress: str | None = None
    port: int | None = None
    ssl: bool = False
    verify_ssl: bool = True
    base_api_path: str | None = None
    url: str | None = None
    api_ver: str | None = None

    def __post_init__(self) -> None:
        """Post init."""
        if self.api_token is None:
            raise ArrException(message="No api token to the server was provided")
        if self.hostname is None and self.ipaddress is None and self.url is None:
            raise ArrException(
                message="No url, hostname or ipaddress to the server was provided"
            )
        HEADERS["X-Api-Key"] = self.api_token

    def api_url(self, command: str, initialize: bool = False) -> str:
        """Return the generated base URL based on host configuration."""
        if initialize:
            return f"{self.base_url}/initialize.js"
        return f"{self.base_url}/api/{self.api_ver}/{command}"

    @property
    def base_url(self) -> str:
        """Return the base URL for the configured service."""
        if self.url is not None:
            return self.url
        protocol = f"http{'s' if self.ssl else ''}"
        host = f"{self.hostname or self.ipaddress}:{self.port}"
        return f"{protocol}://{host}{self.base_api_path or ''}"



"""Sonarr Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from ..const import DATE, EPISODE_ID, PATH, SERIES_ID
from .base import BaseModel
from .request_common import (
    _Common2,
    _Common3,
    _Common4,
    _Common7,
    _Common8,
    _Fields,
    _ImportListCommon,
    _ManualImport,
    _Monitor,
    _MonitorOption,
    _Notification,
    _Quality,
    _QualityCommon,
    _RecordCommon,
    _ReleaseCommon,
    _Rename,
    _StatusMessage,
    _TagDetails,
)
from .sonarr_common import (
    _SonarrAddOptions,
    _SonarrCommon,
    _SonarrCommon2,
    _SonarrEpisodeFile,
    _SonarrEpisodeHistoryData,
    _SonarrEpisodeMonitor,
    _SonarrLanguageItem,
    _SonarrParseEpisodeInfo,
    _SonarrSeries2,
    _SonarrSeriesAlternateTitle,
    _SonarrSeriesCommon,
    _SonarrWantedMissingRecord,
)


class SonarrCommands(str, Enum):
    """Sonarr commands."""

    DOWNLOADED_EPISODES_SCAN = "DownloadedEpisodesScan"
    EPISODE_SEARCH = "EpisodeSearch"
    REFRESH_SERIES = "RefreshSeries"
    RENAME_SERIES = "RenameSeries"
    RESCAN_SERIES = "RescanSeries"
    SEASON_SEARCH = "SeasonSearch"
    SERIES_SEARCH = "SeriesSearch"


class SonarrEventType(Enum):
    """Sonarr event types."""

    DELETED = 5
    FAILED = 4
    GRABBED = 1
    IGNORED = 7
    IMPORTED = 3
    RENAMED = 6


class SonarrSortKeys(str, Enum):
    """Sonarr sort keys."""

    AIR_DATE_UTC = "airDateUtc"
    DATE = DATE
    DOWNLOAD_CLIENT = "downloadClient"
    EPISODE = "episode"
    EPISODE_ID = EPISODE_ID
    EPISODE_TITLE = "episode.title"
    ID = "id"
    INDEXER = "indexer"
    LANGUAGE = "language"
    MESSAGE = "message"
    PATH = PATH
    PROGRESS = "progress"
    PROTOCOL = "protocol"
    QUALITY = "quality"
    RATINGS = "ratings"
    SERIES_ID = SERIES_ID
    SERIES_TITLE = "series.sortTitle"
    SIZE = "size"
    SOURCE_TITLE = "sourcetitle"
    STATUS = "status"
    TIMELEFT = "timeleft"


@dataclass(init=False)
class SonarrCalendar(_SonarrWantedMissingRecord, _SonarrCommon2):
    """Sonarr calendar attributes."""


@dataclass(init=False)
class SonarrEpisode(_SonarrCommon):
    """Sonarr episode attributes."""

    series: type[_SonarrSeriesCommon] = field(default=_SonarrSeriesCommon)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.series = _SonarrSeriesCommon(self.series)


@dataclass(init=False)
class SonarrEpisodeFile(_SonarrEpisodeFile):
    """Sonarr episode file attributes."""


@dataclass(init=False)
class SonarrEpisodeHistory(_Common2, _QualityCommon):
    """Sonarr history record attributes."""

    data: type[_SonarrEpisodeHistoryData] = field(default=_SonarrEpisodeHistoryData)
    date: datetime
    episodeId: int
    id: int
    language: type[_Common3] = field(default=_Common3)
    languageCutoffNotMet: bool
    seriesId: int
    sourceTitle: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.data = _SonarrEpisodeHistoryData(self.data)
        self.language = _Common3(self.language)


@dataclass(init=False)
class SonarrHistory(_RecordCommon):
    """Sonarr history attributes."""

    records: list[SonarrEpisodeHistory] = field(
        default_factory=list[SonarrEpisodeHistory]
    )

    def __post_init__(self):
        """Post init."""
        self.records = [SonarrEpisodeHistory(record) for record in self.records]


@dataclass(init=False)
class SonarrWantedMissing(_RecordCommon):
    """Sonarr wanted missing attributes."""

    records: list[_SonarrWantedMissingRecord] = field(
        default_factory=list[_SonarrWantedMissingRecord]
    )

    def __post_init__(self):
        """Post init."""
        self.records = [_SonarrWantedMissingRecord(record) for record in self.records]


@dataclass(init=False)
class SonarrQueueDetail(_Common4, _Common8):
    """Sonarr queue detail attributes."""

    episode: type[_SonarrCommon] = field(default=_SonarrCommon)
    episodeId: int
    language: type[_Common3] = field(default=_Common3)
    series: type[_SonarrSeries2] = field(default=_SonarrSeries2)
    seriesId: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.quality = _Quality(self.quality)
        self.episode = _SonarrCommon(self.episode)
        self.language = _Common3(self.language)
        self.series = _SonarrSeries2(self.series)
        self.statusMessages = [_StatusMessage(x) for x in self.statusMessages or []]


@dataclass(init=False)
class SonarrQueue(_RecordCommon):
    """Sonarr queue attributes."""

    records: list[SonarrQueueDetail] = field(default_factory=list[SonarrQueueDetail])

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.records = [SonarrQueueDetail(record) for record in self.records]


@dataclass(init=False)
class SonarrParse(BaseModel):
    """Sonarr parse attributes."""

    episodes: list[SonarrEpisode] | None = None
    parsedEpisodeInfo: type[_SonarrParseEpisodeInfo] = field(
        default=_SonarrParseEpisodeInfo
    )
    series: type[_SonarrSeries2] = field(default=_SonarrSeries2)
    title: str

    def __post_init__(self):
        """Post init."""
        self.episodes = [SonarrEpisode(episode) for episode in self.episodes or []]
        self.parsedEpisodeInfo = _SonarrParseEpisodeInfo(self.parsedEpisodeInfo)
        self.series = _SonarrSeries2(self.series)


@dataclass(init=False)
class _SonarrSceneMapping(BaseModel):
    """Sonarr scene mapping attributes."""

    title: str
    seasonNumber: int


@dataclass(init=False)
class SonarrRelease(_ReleaseCommon):
    """Sonarr release attributes."""

    absoluteEpisodeNumbers: list[int]
    episodeNumbers: list[int]
    episodeRequested: bool
    fullSeason: bool
    isAbsoluteNumbering: bool
    isDaily: bool
    isPossibleSpecialEpisode: bool
    language: type[_Common3] = field(default=_Common3)
    languageWeight: int
    mappedAbsoluteEpisodeNumbers: list[int]
    mappedEpisodeNumbers: list[int]
    mappedSeasonNumber: int
    preferredWordScore: int
    quality: type[_Quality] = field(default=_Quality)
    rejected: bool
    releaseGroup: str
    releaseHash: str
    releaseWeight: int
    sceneMapping: type[_SonarrSceneMapping] = field(default=_SonarrSceneMapping)
    seasonNumber: int
    seriesTitle: str
    special: bool
    tvdbId: int
    tvRageId: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language)
        self.quality = _Quality(self.quality)
        self.sceneMapping = _SonarrSceneMapping(self.sceneMapping)


@dataclass(init=False)
class SonarrSeries(_SonarrSeriesCommon):
    """Sonarr series attributes."""

    alternateTitles: list[_SonarrSeriesAlternateTitle] | None = None
    rootFolderPath: str
    previousAiring: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.alternateTitles = [
            _SonarrSeriesAlternateTitle(altTit) for altTit in self.alternateTitles or []
        ]


@dataclass(init=False)
class SonarrSeriesAdd(SonarrSeries):
    """Sonarr series add attributes."""

    addOptions: type[_SonarrAddOptions] = field(default=_SonarrAddOptions)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _SonarrAddOptions(self.addOptions)


@dataclass(init=False)
class SonarrSeriesLookup(_SonarrSeriesCommon):
    """Sonarr series lookup attributes."""


@dataclass(init=False)
class SonarrBlocklistSeries(_Common7):
    """Sonarr blocklist series attributes."""

    date: datetime
    episodeIds: list[int]
    language: type[_Common3] = field(default=_Common3)
    message: str
    quality: type[_Quality] = field(default=_Quality)
    seriesId: int
    sourceTitle: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.language = _Common3(self.language)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class SonarrBlocklist(_RecordCommon):
    """Sonarr blocklist attributes."""

    records: list[SonarrBlocklistSeries] = field(
        default_factory=list[SonarrBlocklistSeries]
    )

    def __post_init__(self):
        """Post init."""
        self.records = [SonarrBlocklistSeries(record) for record in self.records]


@dataclass(init=False)
class SonarrNamingConfig(BaseModel):
    """Sonarr naming config attributes."""

    animeEpisodeFormat: str
    dailyEpisodeFormat: str
    id: int
    includeEpisodeTitle: bool
    includeQuality: bool
    includeSeriesTitle: bool
    multiEpisodeStyle: int
    numberStyle: str
    renameEpisodes: bool
    replaceIllegalCharacters: bool
    replaceSpaces: bool
    seasonFolderFormat: str
    separator: str
    seriesFolderFormat: str
    specialsFolderFormat: str
    standardEpisodeFormat: str


@dataclass(init=False)
class SonarrNotification(_Common3, _Notification):
    """Sonarr notification attributes."""

    fields: list[_Fields] | None = None
    onEpisodeFileDelete: bool
    onEpisodeFileDeleteForUpgrade: bool
    onSeriesDelete: bool
    supportsOnEpisodeFileDelete: bool
    supportsOnEpisodeFileDeleteForUpgrade: bool
    supportsOnSeriesDelete: bool

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class SonarrRename(_Rename):
    """Sonarr rename attributes."""

    episodeFileId: int
    episodeNumbers: list[int]
    seasonNumber: int
    seriesId: int


@dataclass(init=False)
class SonarrTagDetails(_TagDetails):
    """Sonarr tag details attributes."""

    indexerIds: list[int]
    seriesIds: list[int]


@dataclass(init=False)
class SonarrImportList(_ImportListCommon, _Common3):
    """Sonarr importlist attributes."""

    enableAutomaticAdd: bool
    fields: list[_Fields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    languageProfileId: int
    listType: str
    qualityProfileId: int
    seasonFolder: bool
    seriesType: str
    shouldMonitor: str
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class SonarrManualImport(_ManualImport):
    """Sonarr manual import attributes."""

    episodes: list[SonarrEpisodeMonitor] | None = None
    folderName: str
    language: type[_Common3] = field(default=_Common3)
    relativePath: str
    seasonNumber: int
    series: type[_SonarrSeries2] = field(default=_SonarrSeries2)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.episodes = [
            SonarrEpisodeMonitor(episode) for episode in self.episodes or []
        ]
        self.language = _Common3(self.language)
        self.series = _SonarrSeries2(self.series)


@dataclass(init=False)
class SonarrSeasonPass(BaseModel):
    """Sonarr season pass attributes."""

    monitoringOptions: type[_MonitorOption] = field(default=_MonitorOption)
    series: list[_Monitor] | None = None

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.monitoringOptions = _MonitorOption(self.monitoringOptions)
        self.series = [_Monitor(x) for x in self.series or []]


@dataclass(init=False)
class SonarrLanguage(BaseModel):
    """Sonarr launguage attributes."""

    cutoff: type[_Common3] = field(default=_Common3)
    id: int
    languages: list[_SonarrLanguageItem] | None = None
    name: str
    upgradeAllowed: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.cutoff = _Common3(self.cutoff)
        self.languages = [_SonarrLanguageItem(x) for x in self.languages or []]


@dataclass(init=False)
class SonarrEpisodeMonitor(_SonarrEpisodeMonitor):
    """Sonarr episode monitor attributes."""



"""Readarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from .base import BaseModel
from .request_common import (
    _Common3,
    _Common5,
    _Common6,
    _Common7,
    _IsLoaded,
    _Link,
    _Quality,
    _Ratings,
    _SelectOption,
    _TitleInfo,
)


@dataclass(init=False)
class _ReadarrCommon(BaseModel):
    """Readarr Common attributes."""

    authorMetadataId: int
    id: int
    links: list[_Link] | None = None
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        self.links = [_Link(link) for link in self.links or []]


@dataclass(init=False)
class _ReadarrCommon2(BaseModel):
    """Readarr Common attributes."""

    foreignAuthorId: str
    genres: list[str]
    overview: str
    sortName: str
    sortNameLastFirst: str
    status: str


@dataclass(init=False)
class _ReadarrRating(_Ratings):
    """Readarr ratings attributes."""

    popularity: float


@dataclass(init=False)
class _ReadarrAddOptions(BaseModel):
    """Readarr add options attributes."""

    addType: str
    searchForNewBook: bool


@dataclass(init=False)
class _ReadarrImage(_Common5):
    """readarr metadata value attributes."""

    extension: str


@dataclass(init=False)
class _ReadarrMetadataValue(_ReadarrCommon2, _Common3):
    """Readarr metadata value attributes."""

    aliases: list[str]
    born: datetime
    died: datetime
    disambiguation: str
    gender: str
    hometown: str
    images: list[_ReadarrImage] | None = None
    links: list[_Link] | None = None
    nameLastFirst: str
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings)


@dataclass(init=False)
class _ReadarrAuthorMetadata(_IsLoaded):
    """Readarr author Metadata attributes."""

    value: type[_ReadarrMetadataValue] = field(default=_ReadarrMetadataValue)

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrMetadataValue(self.value)


@dataclass(init=False)
class _ReadarrAuthorAddOptions(BaseModel):
    """Readarr author add options attributes."""

    booksToMonitor: list[str]
    monitor: str
    monitored: bool
    searchForMissingBooks: bool


@dataclass(init=False)
class _ReadarrQualityItem(_Common3):
    """Readarr quality item attributes."""

    allowed: bool
    items: list  # Currently unknown contents
    quality: type[_Common3] = field(default=_Common3)

    def __post_init__(self):
        """Post init."""
        self.quality = _Common3(self.quality)


@dataclass(init=False)
class _ReadarrQualityProfileValue(_Common3):
    """Readarr quality profile value attributes."""

    cutoff: int
    items: list[_ReadarrQualityItem] | None = None
    upgradeAllowed: bool

    def __post_init__(self):
        """Post init."""
        self.items = [_ReadarrQualityItem(item) for item in self.items or []]


@dataclass(init=False)
class _ReadarrQualityProfile(_IsLoaded):
    """Readarr quality profile attributes."""

    value: type[_ReadarrQualityProfileValue] = field(
        default=_ReadarrQualityProfileValue
    )

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrQualityProfileValue(self.value)


@dataclass(init=False)
class _ReadarrMetadataProfileValue(_Common3):
    """Readarr metadata profile value attributes."""

    allowedLanguages: str
    ignored: str
    minPages: int
    minPopularity: float
    skipMissingDate: bool
    skipMissingIsbn: bool
    skipPartsAndSets: bool
    skipSeriesSecondary: bool


@dataclass(init=False)
class _ReadarrMetadataProfile(_IsLoaded):
    """Readarr metadata profile attributes."""

    value: type[_ReadarrMetadataProfileValue] = field(
        default=_ReadarrMetadataProfileValue
    )

    def __post_init__(self):
        self.value = _ReadarrMetadataProfileValue(self.value)


@dataclass(init=False)
class _ReadarrAuthorValueBooks(_IsLoaded):
    """Readarr author value books attributes."""

    value: list  # Currently unknown contents


@dataclass(init=False)
class _ReadarrAuthorValueSeriesValue(BaseModel):
    """Readarr author value series value attributes."""

    books: type[_ReadarrAuthorValueBooks] = field(default=_ReadarrAuthorValueBooks)
    description: str
    foreignAuthorId: str
    foreignSeriesId: str
    id: int
    numbered: bool
    primaryWorkCount: int
    title: str
    workCount: int

    def __post_init__(self):
        """Post init."""
        self.books = _ReadarrAuthorValueBooks(self.books)


@dataclass(init=False)
class _ReadarrAuthorValueSeriesLinks(_IsLoaded):
    """Readarr author value series links attributes."""

    value: type[_ReadarrAuthorValueSeriesValue] = field(
        default=_ReadarrAuthorValueSeriesValue
    )

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrAuthorValueSeriesValue(self.value)


@dataclass(init=False)
class _ReadarrAuthorValueSeries(_IsLoaded):
    """Readarr author value series attributes."""

    value: list[_ReadarrAuthorValueSeriesValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrAuthorValueSeriesValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrCommon3(BaseModel):
    """Readarr common attributes."""

    added: datetime
    cleanName: str
    metadataProfileId: int
    monitored: bool
    path: str
    qualityProfileId: int
    tags: list[int]


@dataclass(init=False)
class _ReadarrAuthorValue(_Common3, _ReadarrCommon3):
    """Readarr author value attributes."""

    addOptions: type[_ReadarrAuthorAddOptions] = field(default=_ReadarrAuthorAddOptions)
    authorMetadataId: int
    books: type[_ReadarrAuthorValueBooks] = field(default=_ReadarrAuthorValueBooks)
    foreignAuthorId: str
    lastInfoSync: datetime
    metadata: type[_ReadarrAuthorMetadata] = field(default=_ReadarrAuthorMetadata)
    metadataProfile: type[_ReadarrMetadataProfile] = field(
        default=_ReadarrMetadataProfile
    )
    qualityProfile: type[_ReadarrQualityProfile] = field(default=_ReadarrQualityProfile)
    rootFolderPath: str
    series: type[_ReadarrAuthorValueSeries] = field(default=_ReadarrAuthorValueSeries)

    def __post_init__(self):
        super().__post_init__()
        self.addOptions = _ReadarrAuthorAddOptions(self.addOptions)
        self.books = _ReadarrAuthorValueBooks(self.books)
        self.metadata = _ReadarrAuthorMetadata(self.metadata)
        self.metadataProfile = _ReadarrMetadataProfile(self.metadataProfile)
        self.qualityProfile = _ReadarrQualityProfile(self.qualityProfile)
        self.series = _ReadarrAuthorValueSeries(self.series)


@dataclass(init=False)
class _ReadarrAuthor(_IsLoaded):
    """Readarr author attributes."""

    value: type[_ReadarrAuthorValue] = field(default=_ReadarrAuthorValue)

    def __post_init__(self):
        """Post init."""
        self.value = _ReadarrAuthorValue(self.value)


@dataclass(init=False)
class _ReadarrEditionsValueBookFilesValueMediaInfo(BaseModel):
    """Reaarr editions value book files value media info attributes."""

    audioBitrate: str
    audioBitRate: str
    audioBits: int
    audioChannels: float
    audioCodec: str
    audioFormat: str
    audioSampleRate: str


@dataclass(init=False)
class _ReadarrEditionsValueBookFilesValue(BaseModel):
    """Readarr editions value book files value attributes."""

    author: type[_ReadarrAuthor] = field(default=_ReadarrAuthor)
    calibreId: int
    dateAdded: datetime
    edition: type[_IsLoaded] = field(default=_IsLoaded)
    editionId: int
    id: int
    mediaInfo: type[_ReadarrEditionsValueBookFilesValueMediaInfo] = field(
        default=_ReadarrEditionsValueBookFilesValueMediaInfo
    )
    modified: datetime
    part: int
    partCount: int
    path: str
    quality: type[_Quality] = field(default=_Quality)
    releaseGroup: str
    sceneName: str
    size: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthor(self.author)
        self.edition = _IsLoaded(self.edition)
        self.mediaInfo = _ReadarrEditionsValueBookFilesValueMediaInfo(self.mediaInfo)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _ReadarrEditionsValueBookFiles(_IsLoaded):
    """Readarr editions value book files attributes."""

    value: list[_ReadarrEditionsValueBookFilesValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [
            _ReadarrEditionsValueBookFilesValue(item) for item in self.value or []
        ]


@dataclass(init=False)
class _ReadarrEditionsValue(_Common6):
    """Readarr editions value attributes."""

    asin: str
    book: type[_IsLoaded] = field(default=_IsLoaded)
    bookFiles: type[_ReadarrEditionsValueBookFiles] = field(
        default=_ReadarrEditionsValueBookFiles
    )
    bookId: int
    disambiguation: str
    foreignEditionId: str
    format: str
    grabbed: bool
    id: int
    images: list[_ReadarrImage] | None = None
    isbn13: str
    isEbook: bool
    language: str
    links: list[_Link] | None = None
    manualAdd: bool
    pageCount: int
    publisher: str
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    releaseDate: datetime
    remoteCover: str
    title: str
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.book = _IsLoaded(self.book)
        self.bookFiles = _ReadarrEditionsValueBookFiles(self.bookFiles)
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings)


@dataclass(init=False)
class _ReadarrEditions(_IsLoaded):
    """Readarr editions attributes."""

    value: list[_ReadarrEditionsValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrEditionsValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrSeriesLinksValue(BaseModel):
    """Readarr series links value attributes."""

    book: type[_IsLoaded] = field(default=_IsLoaded)
    bookId: int
    id: int
    isPrimary: bool
    position: str
    series: type[_ReadarrAuthorValueSeriesLinks] = field(
        default=_ReadarrAuthorValueSeriesLinks
    )
    seriesId: int

    def __post_init__(self):
        """Post init."""
        self.book = _IsLoaded(self.book)
        self.series = _ReadarrAuthorValueSeriesLinks(self.series)


@dataclass(init=False)
class _ReadarrSeriesLinks(_IsLoaded):
    """Readarr series links attributes."""

    value: list[_ReadarrSeriesLinksValue] | None = None

    def __post_init__(self):
        """Post init."""
        self.value = [_ReadarrSeriesLinksValue(item) for item in self.value or []]


@dataclass(init=False)
class _ReadarrSeriesLinks2(BaseModel):
    """Readarr series links attributes."""

    bookId: int
    id: int
    position: str
    seriesId: int
    seriesPosition: int


@dataclass(init=False)
class _ReadarrBookCommon(_ReadarrCommon):
    """Readarr book base common attributes."""

    added: datetime
    addOptions: type[_ReadarrAddOptions] = field(default=_ReadarrAddOptions)
    anyEditionOk: bool
    authorMetadata: type[_ReadarrAuthorMetadata] = field(default=_ReadarrAuthorMetadata)
    bookFiles: type[_ReadarrEditionsValueBookFiles] = field(
        default=_ReadarrEditionsValueBookFiles
    )
    cleanTitle: str
    foreignBookId: str
    genres: list[str]
    lastInfoSync: datetime
    monitored: bool
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    releaseDate: datetime
    seriesLinks: type[_ReadarrSeriesLinks] = field(default=_ReadarrSeriesLinks)
    title: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _ReadarrAddOptions(self.addOptions)
        self.authorMetadata = _ReadarrAuthorMetadata(self.authorMetadata)
        self.bookFiles = _ReadarrEditionsValueBookFiles(self.bookFiles)
        self.ratings = _ReadarrRating(self.ratings)
        self.seriesLinks = _ReadarrSeriesLinks(self.seriesLinks)


@dataclass(init=False)
class _ReadarrAuthorStatistics(BaseModel):
    """Readarr author statistics attributes."""

    availableBookCount: int
    bookCount: int
    bookFileCount: int
    percentOfBooks: float
    sizeOnDisk: int
    totalBookCount: int


@dataclass(init=False)
class _ReadarrAuthorBook(_ReadarrBookCommon):
    """Readarr author book attributes."""

    author: type[_ReadarrAuthor] = field(default=_ReadarrAuthor)
    editions: type[_ReadarrEditions] = field(default=_ReadarrEditions)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthor(self.author)
        self.editions = _ReadarrEditions(self.editions)


@dataclass(init=False)
class _ReadarrBlocklistFilter(BaseModel):
    """Readarr blocklist attributes."""

    key: str
    value: str


@dataclass(init=False)
class _ReadarrAuthorBase(_ReadarrCommon2, _ReadarrCommon, _ReadarrCommon3):
    """Readarr author attributes."""

    addOptions: type[_ReadarrAuthorAddOptions] = field(default=_ReadarrAuthorAddOptions)
    authorName: str
    authorNameLastFirst: str
    disambiguation: str
    ended: bool
    images: list[_ReadarrImage] | None = None
    lastBook: type[_ReadarrAuthorBook] = field(default=_ReadarrAuthorBook)
    monitorNewItems: str
    nextBook: type[_ReadarrAuthorBook] = field(default=_ReadarrAuthorBook)
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    remotePoster: str
    rootFolderPath: str
    statistics: type[_ReadarrAuthorStatistics] = field(default=_ReadarrAuthorStatistics)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _ReadarrAuthorAddOptions(self.addOptions)
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.lastBook = _ReadarrAuthorBook(self.lastBook)
        self.nextBook = _ReadarrAuthorBook(self.nextBook)
        self.ratings = _ReadarrRating(self.ratings)
        self.statistics = _ReadarrAuthorStatistics(self.statistics)


@dataclass(init=False)
class _ReadarrBlocklistRecord(_Common7):
    """Readarr blocklist record attributes."""

    author: type[_ReadarrAuthorBase] = field(default=_ReadarrAuthorBase)
    authorId: int
    bookIds: list[int]
    date: datetime
    message: str
    quality: type[_Quality] = field(default=_Quality)
    sourceTitle: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrAuthorBase(self.author)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _ReadarrCountry(BaseModel):
    """Readarr country attributes."""

    name: str
    twoLetterCode: str


@dataclass(init=False)
class _ReadarrAudioTags(BaseModel):
    """Readarr audio tags attributes."""

    asin: str
    authorMBId: str
    authors: list[str]
    authorTitle: str
    bookMBId: str
    bookTitle: str
    catalogNumber: str
    cleanTitle: str
    country: type[_ReadarrCountry] = field(default=_ReadarrCountry)
    disambiguation: str
    discCount: int
    discNumber: int
    duration: str
    goodreadsId: str
    isbn: str
    label: str
    language: str
    mediaInfo: type[_ReadarrEditionsValueBookFilesValueMediaInfo] = field(
        default=_ReadarrEditionsValueBookFilesValueMediaInfo
    )
    publisher: str
    quality: type[_Quality] = field(default=_Quality)
    recordingMBId: str
    releaseGroup: str
    releaseHash: str
    releaseMBId: str
    seriesIndex: str
    seriesTitle: str
    source: str
    title: str
    trackMBId: str
    trackNumbers: list[int]
    year: int

    def __post_init__(self):
        """Post init."""
        self.country = _ReadarrCountry(self.country)
        self.mediaInfo = _ReadarrEditionsValueBookFilesValueMediaInfo(self.mediaInfo)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _ReadarrBookFileMediaInfo(_ReadarrEditionsValueBookFilesValueMediaInfo):
    """Readarr book file media info attributes."""

    id: int


@dataclass(init=False)
class _ReadarrParsedBookInfo(BaseModel):
    """Readarr parsed book info attributes."""

    authorName: str
    authorTitleInfo: type[_TitleInfo] = field(default=_TitleInfo)
    bookTitle: str
    discography: bool
    discographyEnd: int
    discographyStart: int
    quality: type[_Quality] = field(default=_Quality)
    releaseDate: datetime
    releaseGroup: str
    releaseHash: str
    releaseVersion: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.authorTitleInfo = _TitleInfo(self.authorTitleInfo)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _ReadarrSearchAuthor(_ReadarrCommon2, _ReadarrCommon3):
    """Readarr search author attributes."""

    authorMetadataId: int
    authorName: str
    authorNameLastFirst: str
    ended: bool
    id: int
    images: list[_ReadarrImage] | None = None
    links: list[_Link] | None = None
    monitorNewItems: str
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    remotePoster: str
    statistics: type[_ReadarrAuthorStatistics] = field(default=_ReadarrAuthorStatistics)
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings)
        self.statistics = _ReadarrAuthorStatistics(self.statistics)


@dataclass(init=False)
class _ReadarrCategory(_SelectOption):
    """Readarr category attributes."""

    hint: str
    parentValue: int



"""Lidarr Common Models. These are only for internal module use."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from .base import BaseModel
from .request import StatusType
from .request_common import (
    _Common3,
    _Common6,
    _Fields,
    _ImportListCommon,
    _Link,
    _Quality,
    _Ratings,
)


@dataclass(init=False)
class _LidarrMedia(BaseModel):
    """Lidarr release mediaattributes."""

    mediumFormat: str
    mediumName: str
    mediumNumber: int


@dataclass(init=False)
class _LidarrCommon(BaseModel):
    """Lidarr common attributes."""

    albumId: int
    id: int
    qualityCutoffNotMet: bool


@dataclass(init=False)
class _LidarrCommon2(BaseModel):
    """Lidarr album common mediaattributes."""

    images: list[_LidarrImage] | None = None
    links: list[_Link] | None = None
    ratings: type[_Ratings] = field(default=_Ratings)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.images = [_LidarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _Ratings(self.ratings)


@dataclass(init=False)
class _LidarrStatistics(BaseModel):
    """Lidarr statistics attributes."""

    albumCount: int
    percentOfTracks: float
    sizeOnDisk: int
    totalTrackCount: int
    trackCount: int
    trackFileCount: int


@dataclass(init=False)
class _LidarrCommon3(BaseModel):
    """Lidarr common attributes."""

    id: int
    statistics: type[_LidarrStatistics] = field(default=_LidarrStatistics)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.statistics = _LidarrStatistics(self.statistics)


@dataclass(init=False)
class _LidarrCommon4(BaseModel):
    """Lidarr common attributes."""

    disambiguation: str
    foreignAlbumId: str
    genres: list[str]


@dataclass(init=False)
class _LidarrArtist(_LidarrCommon2, _LidarrCommon3, _LidarrCommon4, _Common6):
    """Lidarr artist attributes."""

    added: datetime
    allMusicId: str
    artistMetadataId: int
    artistName: str
    artistType: str
    cleanName: str
    discogsId: int
    ended: bool
    foreignArtistId: str
    mbId: str
    metadataProfileId: int
    path: str
    qualityProfileId: int
    remotePoster: str
    sortName: str
    rootFolderPath: str
    status: StatusType
    tadbId: int
    tags: list[int]


@dataclass(init=False)
class _LidarrCommon5(BaseModel):
    """Lidarr common attributes."""

    artist: type[_LidarrArtist] = field(default=_LidarrArtist)
    artistId: int
    date: datetime
    quality: type[_Quality] = field(default=_Quality)
    sourceTitle: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artist = _LidarrArtist(self.artist)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _LidarrAlbumCommon(_LidarrCommon2, _LidarrCommon4, _Common6):
    """Lidarr album common media attributes."""

    albumType: str
    anyReleaseOk: bool
    artist: type[_LidarrArtist] = field(default=_LidarrArtist)
    artistId: int
    duration: int
    media: list[_LidarrMedia] | None = None
    mediumCount: int
    profileId: int
    releaseDate: datetime
    releases: list[_LidarrRelease] | None = None
    secondaryTypes: list[_Common3] | None = None
    title: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artist = _LidarrArtist(self.artist)
        self.media = [_LidarrMedia(x) for x in self.media or []]
        self.releases = [_LidarrRelease(release) for release in self.releases or []]
        self.secondaryTypes = [_Common3(x) for x in self.secondaryTypes or []]


@dataclass(init=False)
class _LidarrRelease(BaseModel):
    """Lidarr release attributes."""

    albumId: int
    country: list[str]
    disambiguation: str
    duration: int
    foreignReleaseId: str
    format: str
    id: int
    label: str
    media: list[_LidarrMedia] | None = None
    mediumCount: int
    monitored: bool
    status: str
    title: str
    trackCount: int

    def __post_init__(self):
        """Post init."""
        self.media = [_LidarrMedia(x) for x in self.media or []]


@dataclass(init=False)
class _LidarrImage(BaseModel):
    """Lidarr image attributes."""

    coverType: str
    extension: str
    url: str


@dataclass(init=False)
class _LidarrFields(_Fields):
    """Lidarr fields attributes."""

    selectOptionsProviderAction: str


@dataclass(init=False)
class _LidarrImportListPreset(_ImportListCommon):
    """Lidarr import list preset attributes."""

    enableAutomaticAdd: bool
    fields: list[_LidarrFields] | None = None
    implementation: str
    infoLink: str
    listType: str
    metadataProfileId: int
    name: str
    qualityProfileId: int
    shouldMonitor: str
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.fields = [_LidarrFields(field) for field in self.fields or []]


@dataclass(init=False)
class _LidarrArtistTitleInfo(BaseModel):
    """Lidarr artist title info attributes."""

    title: str
    year: int


@dataclass(init=False)
class _LidarrMediaInfo(BaseModel):
    """Lidarr media info attributes."""

    audioBitrate: str
    audioBitRate: str
    audioBits: str
    audioChannels: int | float
    audioCodec: str
    audioFormat: str
    audioSampleRate: str


@dataclass(init=False)
class _LidarrMediaInfo_Quality(BaseModel):
    """Lidarr media info/quality attributes."""

    mediaInfo: type[_LidarrMediaInfo] = field(default=_LidarrMediaInfo)
    quality: type[_Quality] = field(default=_Quality)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.mediaInfo = _LidarrMediaInfo(self.mediaInfo)
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class _LidarrAudioTags(_LidarrMediaInfo_Quality, _LidarrArtistTitleInfo):
    """Lidarr audio tags attributes."""

    albumTitle: str
    artistTitle: str
    artistTitleInfo: type[_LidarrArtistTitleInfo] = field(
        default=_LidarrArtistTitleInfo
    )
    cleanTitle: str
    discCount: int
    discNumber: int
    duration: str
    trackNumbers: list[int]

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.artistTitleInfo = _LidarrArtistTitleInfo(self.artistTitleInfo)


@dataclass(init=False)
class _LidarrAddOptions(BaseModel):
    """Lidarr add options attributes."""

    addType: str
    searchForNewAlbum: bool



"""Readarr Models."""
# pylint: disable=invalid-name, too-many-instance-attributes
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from ..const import AUTHOR_ID, PATH, TITLE
from .base import BaseModel
from .readarr_common import (
    _ReadarrAddOptions,
    _ReadarrAudioTags,
    _ReadarrAuthorAddOptions,
    _ReadarrAuthorBase,
    _ReadarrAuthorStatistics,
    _ReadarrBlocklistFilter,
    _ReadarrBlocklistRecord,
    _ReadarrBookCommon,
    _ReadarrBookFileMediaInfo,
    _ReadarrCategory,
    _ReadarrEditionsValue,
    _ReadarrImage,
    _ReadarrMetadataProfileValue,
    _ReadarrParsedBookInfo,
    _ReadarrRating,
    _ReadarrSearchAuthor,
    _ReadarrSeriesLinks2,
)
from .request_common import (
    _Common2,
    _Common3,
    _Common4,
    _Common6,
    _Common8,
    _Editor,
    _Fields,
    _HistoryData,
    _ImportListCommon,
    _Link,
    _ManualImport,
    _Notification,
    _Quality,
    _QualityCommon,
    _RecordCommon,
    _ReleaseCommon,
    _Rename,
    _RetagChange,
    _StatusMessage,
    _TagDetails,
)


class ReadarrBookTypes(str, Enum):
    """Readarr book types."""

    ASIN = "asin"
    GOODREADS = "goodreads"
    ISBN = "isbn"


class ReadarrCommands(str, Enum):
    """Readarr commands."""

    APP_UPDATE_CHECK = "ApplicationUpdateCheck"
    AUTHOR_SEARCH = "AuthorSearch"
    BOOK_SEARCH = "BookSearch"
    REFRESH_AUTHOR = "RefreshAuthor"
    REFRESH_BOOK = "RefreshBook"
    RENAME_AUTHOR = "RenameAuthor"
    RESCAN_FOLDERS = "RescanFolders"


class ReadarrEventType(Enum):
    """Readarr event types."""

    BOOK_IMPORTED = 3
    DELETED = 5
    DOWNLOAD_FAILED = 4
    DOWNLOAD_IMPORTED = 8
    GRABBED = 1
    IGNORED = 7
    IMPORT_FAILED = 7
    RENAMED = 6
    RETAGGED = 9


class ReadarrImportListType(str, Enum):
    """Readarr import list types."""

    PROGRAM = "program"
    GOODREADS = "goodreads"
    OTHER = "other"


class ReadarrSortKeys(str, Enum):
    """Readarr sort keys."""

    AUTHOR_ID = AUTHOR_ID
    BOOK_ID = "Books.Id"
    DATE = "books.releaseDate"
    DOWNLOAD_CLIENT = "downloadClient"
    ID = "id"
    INDEXER = "indexer"
    MESSAGE = "message"
    PATH = PATH
    PROGRESS = "progress"
    PROTOCOL = "protocol"
    QUALITY = "quality"
    RATINGS = "ratings"
    SIZE = "size"
    SOURCE_TITLE = "sourcetitle"
    STATUS = "status"
    TIMELEFT = "timeleft"
    TITLE = TITLE


@dataclass(init=False)
class ReadarrAuthor(_ReadarrAuthorBase):
    """Readarr author attributes."""


@dataclass(init=False)
class ReadarrBook(_ReadarrBookCommon):
    """Readarr book attributes."""

    author: type[ReadarrAuthor] = field(default=ReadarrAuthor)
    authorId: int
    authorTitle: str
    disambiguation: str
    editions: list[_ReadarrEditionsValue] | None = None
    grabbed: bool
    images: list[_ReadarrImage] | None = None
    overview: str
    pageCount: int
    remoteCover: str
    seriesTitle: str
    statistics: type[_ReadarrAuthorStatistics] = field(default=_ReadarrAuthorStatistics)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = ReadarrAuthor(self.author)
        self.editions = [_ReadarrEditionsValue(x) for x in self.editions or []]
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.statistics = _ReadarrAuthorStatistics(self.statistics)


@dataclass(init=False)
class ReadarrAuthorLookup(ReadarrAuthor):
    """Readarr author attributes."""

    monitorNewItems: str


@dataclass(init=False)
class ReadarrBlocklist(_RecordCommon):
    """Readarr blocklist attributes."""

    filters: list[_ReadarrBlocklistFilter] | None = None
    records: list[_ReadarrBlocklistRecord] = field(
        default_factory=list[_ReadarrBlocklistRecord]
    )

    def __post_init__(self):
        """Post init."""
        self.filters = [
            _ReadarrBlocklistFilter(filter) for filter in self.filters or []
        ]
        self.records = [_ReadarrBlocklistRecord(record) for record in self.records]


@dataclass(init=False)
class ReadarrAuthorEditor(_Editor):
    """Readarr author editor attributes."""

    authorIds: list[int]
    metadataProfileId: int


@dataclass(init=False)
class ReadarrBookFile(_QualityCommon):
    """Readarr book file attributes."""

    audioTags: type[_ReadarrAudioTags] = field(default=_ReadarrAudioTags)
    authorId: int
    bookId: int
    dateAdded: datetime
    id: int
    mediaInfo: type[_ReadarrBookFileMediaInfo] = field(
        default=_ReadarrBookFileMediaInfo
    )
    path: str
    qualityWeight: int
    size: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.audioTags = _ReadarrAudioTags(self.audioTags)
        self.mediaInfo = _ReadarrBookFileMediaInfo(self.mediaInfo)


@dataclass(init=False)
class ReadarrBookFileEditor(BaseModel):
    """Readarr book file attributes."""

    bookFileIds: list[int]
    quality: type[_Quality] = field(default=_Quality)

    def __post_init__(self):
        """Post init."""
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class ReadarrBookLookup(_Common6):
    """Readarr book lookup attributes."""

    added: datetime
    anyEditionOk: bool
    author: type[ReadarrAuthor] = field(default=ReadarrAuthor)
    authorId: int
    authorTitle: str
    disambiguation: str
    editions: list[_ReadarrEditionsValue] | None = None
    foreignBookId: str
    genres: list[str]
    grabbed: bool
    images: list[_ReadarrImage] | None = None
    links: list[_Link] | None = None
    pageCount: int
    ratings: type[_ReadarrRating] = field(default=_ReadarrRating)
    releaseDate: datetime
    remoteCover: str
    seriesTitle: str
    title: str
    titleSlug: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = ReadarrAuthor(self.author)
        self.editions = [_ReadarrEditionsValue(editn) for editn in self.editions or []]
        self.images = [_ReadarrImage(image) for image in self.images or []]
        self.links = [_Link(link) for link in self.links or []]
        self.ratings = _ReadarrRating(self.ratings)


@dataclass(init=False)
class ReadarrBookshelfAuthorBook(ReadarrBookLookup):
    """Readarr bookshelf author Book attributes."""

    addOptions: type[_ReadarrAddOptions] = field(default=_ReadarrAddOptions)
    id: int
    statistics: type[_ReadarrAuthorStatistics] = field(default=_ReadarrAuthorStatistics)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.addOptions = _ReadarrAddOptions(self.addOptions)
        self.statistics = _ReadarrAuthorStatistics(self.statistics)


@dataclass(init=False)
class ReadarrCalendar(ReadarrBookshelfAuthorBook):
    """Readarr calendar attributes."""


@dataclass(init=False)
class ReadarrBookshelfAuthor(BaseModel):
    """Readarr bookshelf author attributes."""

    books: list[ReadarrBookshelfAuthorBook] | None = None
    id: int
    monitored: bool

    def __post_init__(self):
        """Post init."""
        self.books = [ReadarrBookshelfAuthorBook(book) for book in self.books or []]


@dataclass(init=False)
class ReadarrBookshelf(BaseModel):
    """Readarr bookshelf attributes."""

    authors: list[ReadarrBookshelfAuthor] | None = None
    monitoringOptions: type[_ReadarrAuthorAddOptions] = field(
        default=_ReadarrAuthorAddOptions
    )

    def __post_init__(self):
        """Post init."""
        self.authors = [ReadarrBookshelfAuthor(author) for author in self.authors or []]
        self.monitoringOptions = _ReadarrAuthorAddOptions(self.monitoringOptions)


@dataclass(init=False)
class ReadarrWantedMissing(_RecordCommon):
    """Readarr wanted missing attributes."""

    records: list[ReadarrBook] = field(default_factory=list[ReadarrBook])

    def __post_init__(self):
        """Post init."""
        self.records = [ReadarrBook(record) for record in self.records]


@dataclass(init=False)
class ReadarrWantedCutoff(ReadarrWantedMissing):
    """Readarr wanted cutoff attributes."""

    filters: list[_ReadarrBlocklistFilter] | None = None

    def __post_init__(self):
        """Post init."""
        self.filters = [
            _ReadarrBlocklistFilter(filter) for filter in self.filters or []
        ]


@dataclass(init=False)
class ReadarrMetadataProfile(_ReadarrMetadataProfileValue):
    """Readarr metadata profile attributes."""


@dataclass(init=False)
class ReadarrDevelopmentConfig(BaseModel):
    """Readarr development config attributes."""

    consoleLogLevel: str
    filterSentryEvents: bool
    id: int
    logRotate: int
    logSql: bool
    metadataSource: str


@dataclass(init=False)
class ReadarrBookHistory(_Common2, _QualityCommon):
    """Readarr history record attributes."""

    authorId: int
    bookId: int
    data: type[_HistoryData] = field(default=_HistoryData)
    date: datetime
    id: int
    sourceTitle: str

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.data = _HistoryData(self.data)


@dataclass(init=False)
class ReadarrHistory(_RecordCommon):
    """Readarr history attributes."""

    records: list[ReadarrBookHistory] = field(default_factory=list[ReadarrBookHistory])

    def __post_init__(self):
        """Post init."""
        self.records = [ReadarrBookHistory(record) for record in self.records]


@dataclass(init=False)
class ReadarrImportList(_ImportListCommon, _Common3):
    """Readarr importlist attributes."""

    enableAutomaticAdd: bool
    fields: list[_Fields] | None = None
    implementation: str
    implementationName: str
    infoLink: str
    listType: str
    metadataProfileId: int
    monitorNewItems: str
    qualityProfileId: int
    shouldMonitor: str
    shouldMonitorExisting: bool
    shouldSearch: bool
    tags: list[int]

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class ReadarrMetadataProvider(BaseModel):
    """Readarr metadata provider attributes."""

    embedMetadata: bool
    id: int
    scrubAudioTags: bool
    updateCovers: bool
    writeAudioTags: str
    writeBookTags: str


@dataclass(init=False)
class ReadarrNamingConfig(BaseModel):
    """Readarr naming config attributes."""

    authorFolderFormat: str
    id: int
    includeAuthorName: bool
    includeBookTitle: bool
    includeQuality: bool
    renameBooks: bool
    replaceIllegalCharacters: bool
    replaceSpaces: bool
    standardBookFormat: str


@dataclass(init=False)
class ReadarrNotification(_Common3, _Notification):
    """Readarr notification attributes."""

    fields: list[_Fields] | None = None
    onBookRetag: bool
    onDownloadFailure: bool
    onImportFailure: bool
    onReleaseImport: bool
    supportsOnBookRetag: bool
    supportsOnDownloadFailure: bool
    supportsOnImportFailure: bool
    supportsOnReleaseImport: bool

    def __post_init__(self):
        """Post init."""
        self.fields = [_Fields(field) for field in self.fields or []]


@dataclass(init=False)
class ReadarrParse(BaseModel):
    """Readarr parse attributes."""

    author: type[ReadarrAuthor] = field(default=ReadarrAuthor)
    books: list[ReadarrBook] | None = None
    id: int
    parsedBookInfo: type[_ReadarrParsedBookInfo] = field(default=_ReadarrParsedBookInfo)
    title: str

    def __post_init__(self):
        """Post init."""
        self.author = ReadarrAuthor(self.author)
        self.books = [ReadarrBook(book) for book in self.books or []]
        self.parsedBookInfo = _ReadarrParsedBookInfo(self.parsedBookInfo)


@dataclass(init=False)
class ReadarrQueueDetail(_Common4, _Common8):
    """Readarr queue detail attributes."""

    author: type[ReadarrAuthor] = field(default=ReadarrAuthor)
    authorId: int
    book: type[ReadarrBook] = field(default=ReadarrBook)
    bookId: int
    downloadForced: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = ReadarrAuthor(self.authorId)
        self.book = ReadarrBook(self.book)
        self.quality = _Quality(self.quality)
        self.statusMessages = [_StatusMessage(x) for x in self.statusMessages or []]


@dataclass(init=False)
class ReadarrQueue(_RecordCommon):
    """Readarr queue attributes."""

    records: list[ReadarrQueueDetail] = field(default_factory=list[ReadarrQueueDetail])

    def __post_init__(self):
        """Post init."""
        self.records = [ReadarrQueueDetail(record) for record in self.records]


@dataclass(init=False)
class ReadarrRelease(_ReleaseCommon):
    """Readarr release attributes."""

    authorName: str
    bookTitle: str
    discography: bool
    preferredWordScore: int
    quality: type[_Quality] = field(default=_Quality)

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.quality = _Quality(self.quality)


@dataclass(init=False)
class ReadarrRename(_Rename):
    """Readarr rename attributes."""

    authorId: int
    bookFileId: int
    bookId: int


@dataclass(init=False)
class ReadarrRetag(ReadarrRename):
    """Readarr retag attributes."""

    changes: list[_RetagChange] | None = None
    path: str
    trackNumbers: list[int]

    def __post_init__(self):
        """Post init."""
        self.changes = [_RetagChange(change) for change in self.changes or []]


@dataclass(init=False)
class ReadarrSearch(BaseModel):
    """Readarr search attributes."""

    author: type[_ReadarrSearchAuthor] = field(default=_ReadarrSearchAuthor)
    foreignId: str
    id: int

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.author = _ReadarrSearchAuthor(self.author)


@dataclass(init=False)
class ReadarrSeries(BaseModel):
    """Readarr series attributes."""

    description: str
    id: int
    links: list[_ReadarrSeriesLinks2] | None = None
    title: str

    def __post_init__(self):
        """Post init."""
        self.links = [_ReadarrSeriesLinks2(link) for link in self.links or []]


@dataclass(init=False)
class ReadarrTagDetails(_TagDetails):
    """Readarr tag details attributes."""

    authorIds: list[int]


@dataclass(init=False)
class ReadarrManualImport(_ManualImport):
    """Readarr manual import attributes."""

    additionalFile: bool
    audioTags: type[_ReadarrAudioTags] = field(default=_ReadarrAudioTags)
    author: type[ReadarrAuthor] = field(default=ReadarrAuthor)
    book: type[ReadarrBook] = field(default=ReadarrBook)
    disableReleaseSwitching: bool
    foreignEditionId: int
    replaceExistingFiles: bool

    def __post_init__(self):
        """Post init."""
        super().__post_init__()
        self.audioTags = _ReadarrAudioTags(self.audioTags)
        self.author = ReadarrAuthor(self.author)
        self.book = ReadarrBook(self.book)


@dataclass(init=False)
class ReadarrImportListOptions(BaseModel):
    """Readarr import list options attributes."""

    options: list[_ReadarrCategory] | None = None

    def __post_init__(self):
        """Post init."""
        self.options = [_ReadarrCategory(option) for option in self.options or []]



"""Lidarr API."""
from __future__ import annotations

from datetime import datetime
from typing import Any

from aiohttp.client import ClientSession

from aiopyarr.exceptions import ArrException

from .const import (
    ALBUM_ID,
    ALL,
    ARTIST_ID,
    DATE,
    EVENT_TYPE,
    IS_VALID,
    PAGE,
    PAGE_SIZE,
    SORT_DIRECTION,
    SORT_KEY,
    TERM,
    TITLE,
    HTTPMethod,
)
from .models.host_configuration import PyArrHostConfiguration
from .models.lidarr import (
    LidarrAlbum,
    LidarrAlbumEditor,
    LidarrAlbumHistory,
    LidarrAlbumLookup,
    LidarrAlbumStudio,
    LidarrArtist,
    LidarrArtistEditor,
    LidarrBlocklist,
    LidarrCalendar,
    LidarrCommands,
    LidarrEventType,
    LidarrHistory,
    LidarrImportList,
    LidarrImportListActionType,
    LidarrManualImport,
    LidarrMetadataProfile,
    LidarrMetadataProvider,
    LidarrParse,
    LidarrQueue,
    LidarrQueueItem,
    LidarrRelease,
    LidarrRename,
    LidarrRetag,
    LidarrSearch,
    LidarrSortKeys,
    LidarrTagDetails,
    LidarrTrack,
    LidarrTrackFile,
    LidarrTrackFileEditor,
    LidarrWantedCutoff,
)
from .models.request import Command, SortDirection
from .request_client import RequestClient


class LidarrClient(RequestClient):  # pylint: disable=too-many-public-methods
    """API client for Lidarr endpoints."""

    __name__ = "Lidarr"

    def __init__(  # pylint: disable=too-many-arguments
        self,
        host_configuration: PyArrHostConfiguration | None = None,
        session: ClientSession | None = None,
        hostname: str | None = None,
        ipaddress: str | None = None,
        url: str | None = None,
        api_token: str | None = None,
        port: int = 8686,
        ssl: bool | None = None,
        verify_ssl: bool | None = None,
        base_api_path: str | None = None,
        request_timeout: float = 10,
        raw_response: bool = False,
        api_ver: str = "v1",
    ) -> None:
        """Initialize Lidarr API."""
        super().__init__(
            port,
            request_timeout,
            raw_response,
            api_ver,
            host_configuration,
            session,
            hostname,
            ipaddress,
            url,
            api_token,
            ssl,
            verify_ssl,
            base_api_path,
        )

    async def async_get_albums(
        self,
        albumids: int | list[int] | None = None,
        artistid: int | None = None,
        foreignalbumid: int | None = None,
        allartistalbums: bool = False,
    ) -> LidarrAlbum | list[LidarrAlbum]:
        """Get info about specified album by id, leave blank for all."""
        params: dict[str, str | int | list[int]] = {}
        params["includeAllArtistAlbums"] = str(allartistalbums)
        if isinstance(albumids, list):
            params["albumids"] = albumids
        if artistid is not None:
            params[ARTIST_ID] = artistid
        if foreignalbumid is not None:
            params["foreignAlbumId"] = foreignalbumid
        cmd = "" if isinstance(albumids, list) or albumids is None else f"/{albumids}"
        return await self._async_request(
            f"album{cmd}",
            params=params,
            datatype=LidarrAlbum,
        )

    async def async_add_album(self, data: LidarrAlbum) -> LidarrAlbum:
        """Add album to database."""
        return await self._async_request(
            "album", data=data, datatype=LidarrAlbum, method=HTTPMethod.POST
        )

    async def async_edit_albums(
        self, data: LidarrAlbum | LidarrAlbumEditor
    ) -> LidarrAlbum | list[LidarrAlbum]:
        """Edit album database info."""
        return await self._async_request(
            f"album{'' if isinstance(data, LidarrAlbum) else '/monitor'}",
            data=data,
            datatype=LidarrAlbum,
            method=HTTPMethod.PUT,
        )

    async def async_delete_album(self, albumid: int) -> None:
        """Delete the album with the given id."""
        return await self._async_request(f"album/{albumid}", method=HTTPMethod.DELETE)

    async def async_album_studio(self, data: LidarrAlbumStudio) -> None:
        """Change monitoring status via album studio."""
        return await self._async_request(
            "albumstudio",
            data=data,
            method=HTTPMethod.POST,
        )

    async def async_get_artists(
        self,
        entryid: str | int | None = None,
    ) -> LidarrArtist | list[LidarrArtist]:
        """Get info about specified artists by id, leave blank for all.

        entryid: Include a string to search by MusicBrainz id.
        """
        command = "" if isinstance(entryid, str) or entryid is None else f"/{entryid}"
        return await self._async_request(
            f"artist{command}",
            params={"mbId": entryid} if isinstance(entryid, str) else None,
            datatype=LidarrArtist,
        )

    async def async_add_artist(self, data: LidarrArtist) -> LidarrArtist:
        """Add artist to database."""
        return await self._async_request(
            "artist", data=data, datatype=LidarrArtist, method=HTTPMethod.POST
        )

    async def async_edit_artists(
        self, data: LidarrArtist | LidarrArtistEditor
    ) -> LidarrArtist | list[LidarrArtist]:
        """Edit artist database info."""
        return await self._async_request(
            f"artist{'' if isinstance(data, LidarrArtist) else '/editor'}",
            data=data,
            datatype=LidarrArtist,
            method=HTTPMethod.PUT,
        )

    async def async_delete_artists(self, data: int | dict) -> None:
        """Delete the artists with the given ids."""
        return await self._async_request(
            f"artist/{data if isinstance(data, int) else 'editor'}",
            data=data if not isinstance(data, int) else None,
            method=HTTPMethod.DELETE,
        )

    async def async_album_lookup(self, term: str) -> list[LidarrAlbumLookup]:
        """Search for new albums using a term."""
        return await self._async_request(
            "album/lookup", params={TERM: term}, datatype=LidarrAlbumLookup
        )

    # artist/import POST not confirmed

    async def async_get_blocklist(
        self,
        page: int = 1,
        page_size: int = 10,
        sort_dir: SortDirection = SortDirection.DEFAULT,
        sort_key: LidarrSortKeys = LidarrSortKeys.DATE,
    ) -> LidarrBlocklist:
        """Return blocklisted releases.

        Args:
            page: Page to be returned.
            page_size: Number of results per page.
            sort_key: date, id, artistid, sourcetitle, quality,
            indexer, path, or message. (Others do not apply)
        """
        params = {
            PAGE: page,
            PAGE_SIZE: page_size,
            SORT_DIRECTION: sort_dir.value,
            SORT_KEY: sort_key.value,
        }
        return await self._async_request(
            "blacklist",
            params=params,
            datatype=LidarrBlocklist,
        )

    async def async_get_calendar(  # pylint: disable=too-many-arguments
        self,
        start_date: datetime | None = None,
        end_date: datetime | None = None,
        albumid: int | None = None,
        unmonitored: bool = False,
    ) -> LidarrCalendar | list[LidarrCalendar]:
        """Get calendar items.

        albumid: Specify to get calendar of releases from album id
        """
        params = {"unmonitored": str(unmonitored)}
        if start_date:
            params["start"] = start_date.strftime("%Y-%m-%d")
        if end_date:
            params["end"] = end_date.strftime("%Y-%m-%d")
        return await self._async_request(
            f"calendar{'' if albumid is None else f'/{albumid}'}",
            params=params,
            datatype=LidarrCalendar,
        )

    async def async_lidarr_command(self, command: LidarrCommands) -> Command:
        """Send a command to Lidarr."""
        return await self._async_request(
            "command",
            data={"name": command.value},
            datatype=Command,
            method=HTTPMethod.POST,
        )

    async def async_get_wanted(  # pylint: disable=too-many-arguments
        self,
        recordid: int | None = None,
        sort_key: LidarrSortKeys = LidarrSortKeys.TITLE,
        page: int = 1,
        page_size: int = 10,
        missing: bool = True,
    ) -> LidarrWantedCutoff | LidarrAlbum:
        """Get wanted albums not meeting cutoff or missing.

        Args:
            sort_key: id, title, ratings, or quality". (Others do not apply)
            page: Page number to return.
            page_size: Number of items per page.
            missing: Search for missing or cutoff not met
        """
        params = {
            SORT_KEY: sort_key.value,
            PAGE: page,
            PAGE_SIZE: page_size,
        }
        _cmd = "missing" if missing else "cutoff"
        return await self._async_request(
            f"wanted/{_cmd}{'' if recordid is None else f'/{recordid}'}",
            params=params,
            datatype=LidarrWantedCutoff if recordid is None else LidarrAlbum,
        )

    async def async_parse(self, title: str) -> LidarrParse:
        """Return the movie with matching file name."""
        params = {TITLE: title}
        return await self._async_request("parse", params=params, datatype=LidarrParse)

    async def async_get_history(  # pylint: disable=too-many-arguments
        self,
        page: int = 1,
        pagesize: int = 10,
        sort_key: LidarrSortKeys = LidarrSortKeys.DATE,
        sort_dir: SortDirection = SortDirection.DEFAULT,
        event_type: LidarrEventType | None = None,
        artist: bool = False,
        album: bool = False,
    ) -> LidarrHistory:
        """Get history.

        sort_key: date, quality, title, id, artistid, path, ratings, or sourcetitle
                (Others do not apply)
        """
        params = {
            PAGE: page,
            PAGE_SIZE: pagesize,
            SORT_KEY: sort_key.value,
            SORT_DIRECTION: sort_dir.value,
        }
        if event_type and event_type in LidarrEventType:
            params[EVENT_TYPE] = event_type.value
        if artist is not None:
            params["includeArtist"] = str(artist)
        if album is not None:
            params["includeAlbum"] = str(album)
        return await self._async_request(
            "history",
            params=params,
            datatype=LidarrHistory,
        )

    async def async_get_history_since(
        self,
        date: datetime | None = None,
        artistid: int | None = None,
        event_type: LidarrEventType | None = None,
    ) -> list[LidarrAlbumHistory]:
        """Get history since specified date.

        artist: True, include artist info in response
                int, search history by artist
        album: True, include album info in response
                int, specify album when artist: int is specified
        date: Specify a datetime object to get history since that date
        """
        if date is None and artistid is None:
            raise ArrException(self, "Either date or artistid is required")
        params: dict[str, int | str] = {}
        if event_type and event_type in LidarrEventType:
            params[EVENT_TYPE] = event_type.value
        if isinstance(date, datetime):
            params[DATE] = date.strftime("%Y-%m-%d")
        elif artistid is not None:
            params[ARTIST_ID] = artistid
        return await self._async_request(
            f"history/{'since' if isinstance(date, datetime) else 'artist'}",
            params=params,
            datatype=LidarrAlbumHistory,
        )

    async def async_get_import_lists(
        self, listid: int | None = None
    ) -> LidarrImportList | list[LidarrImportList]:
        """Get import lists."""
        return await self._async_request(
            f"importlist{'' if listid is None else f'/{listid}'}",
            datatype=LidarrImportList,
        )

    async def async_edit_import_list(self, data: LidarrImportList) -> LidarrImportList:
        """Edit import list."""
        return await self._async_request(
            "importlist",
            data=data,
            datatype=LidarrImportList,
            method=HTTPMethod.PUT,
        )

    async def async_add_import_list(self, data: LidarrImportList) -> LidarrImportList:
        """Add import list."""
        return await self._async_request(
            "importlist", data=data, datatype=LidarrImportList, method=HTTPMethod.POST
        )

    async def async_test_import_lists(
        self, data: LidarrImportList | None = None
    ) -> bool:
        """Test all import lists."""
        _res = await self._async_request(
            f"importlist/test{ALL if data is None else ''}",
            data=None if data is None else data,
            method=HTTPMethod.POST,
        )
        if data is None:
            for item in _res:
                if item[IS_VALID] is False:
                    return False
        return True

    async def async_importlist_action(
        self, action: LidarrImportListActionType, data: LidarrImportList | None = None
    ) -> dict[str, Any]:
        """Perform import list action."""
        if action is LidarrImportListActionType.GET_PLAYLISTS and data is None:
            raise ArrException(self, "Data is required when calling getPlaylists")
        return await self._async_request(
            f"importlist/action/{action.value}",
            data=data,
            method=HTTPMethod.POST,
        )

    async def async_get_metadata_profiles(
        self, profileid: int | None = None
    ) -> LidarrMetadataProfile | list[LidarrMetadataProfile]:
        """Get metadata profiles."""
        return await self._async_request(
            f"metadataprofile{'' if profileid is None else f'/{profileid}'}",
            datatype=LidarrMetadataProfile,
        )

    async def async_edit_metadata_profile(
        self, data: LidarrMetadataProfile
    ) -> LidarrMetadataProfile:
        """Edit a metadata profile."""
        return await self._async_request(
            "metadataprofile",
            data=data,
            datatype=LidarrMetadataProfile,
            method=HTTPMethod.PUT,
        )

    async def async_add_metadata_profile(
        self, data: LidarrMetadataProfile
    ) -> LidarrMetadataProfile:
        """Add a metadata profile."""
        return await self._async_request(
            "metadataprofile",
            data=data,
            datatype=LidarrMetadataProfile,
            method=HTTPMethod.POST,
        )

    async def async_get_metadata_provider(self) -> LidarrMetadataProvider:
        """Get metadata provider."""
        return await self._async_request(
            "config/metadataprovider",
            datatype=LidarrMetadataProvider,
        )

    async def async_edit_metadata_provider(
        self, data: LidarrMetadataProvider
    ) -> LidarrMetadataProvider:
        """Edit metadata provider."""
        return await self._async_request(
            "config/metadataprovider",
            data=data,
            datatype=LidarrMetadataProvider,
            method=HTTPMethod.PUT,
        )

    async def async_get_queue(  # pylint: disable=too-many-arguments
        self,
        page: int = 1,
        page_size: int = 10,
        sort_key: LidarrSortKeys = LidarrSortKeys.TIMELEFT,
        unknown_artists: bool = False,
        include_artist: bool = False,
        include_album: bool = False,
    ) -> LidarrQueue:
        """Get current download information.

        Args:
            page: page number
            page_size: number of results per page_size
            unknown_artists: Include items with an unknown artist
            include_artist: Include the artist
            include_album: Include the album
        """
        params = {
            PAGE: page,
            PAGE_SIZE: page_size,
            SORT_KEY: sort_key.value,
            "includeUnknownArtistItems": str(unknown_artists),
            "includeArtist": str(include_artist),
            "includeAlbum": str(include_album),
        }
        return await self._async_request("queue", params=params, datatype=LidarrQueue)

    async def async_get_queue_details(  # pylint: disable=too-many-arguments
        self,
        artistid: int | None = None,
        albumids: list[int] | None = None,
        include_artist: bool = False,
        include_album: bool = True,
    ) -> list[LidarrQueueItem]:
        """Get details of all items in queue."""
        params: dict = {
            "includeArtist": str(include_artist),
            "includeAlbum": str(include_album),
        }
        if artistid is not None:
            params[ARTIST_ID] = artistid
        if albumids is not None:
            params["albumIds"] = albumids
        return await self._async_request(
            "queue/details",
            params=params,
            datatype=LidarrQueueItem,
        )

    async def async_get_release(
        self, artistid: int | None = None, albumid: int | None = None
    ) -> list[LidarrRelease]:
        """Search indexers for specified fields."""
        params = {}
        if artistid is not None:
            params[ARTIST_ID] = artistid
        if albumid is not None:
            params[ALBUM_ID] = albumid
        return await self._async_request(
            "release", params=params, datatype=LidarrRelease
        )

    async def async_download_release(
        self, guid: str, indexerid: int
    ) -> list[LidarrRelease]:
        """Add a previously searched release to the download client.

        If the release is
        still in the search cache (30 minute cache). If the release is not found
        in the cache it will return a 404.

        guid: Recently searched result guid
        """
        return await self._async_request(
            "release",
            data={"guid": guid, "indexerId": indexerid},
            datatype=LidarrRelease,
            method=HTTPMethod.POST,
        )

    # Only works if an id is associated with the release
    async def async_get_pushed_release(self, releaseid: str) -> LidarrRelease:
        """Get release previously pushed by below method."""
        return await self._async_request(
            "release/push",
            params={"id": releaseid},
            datatype=LidarrRelease,
        )

    async def async_get_rename(
        self, artistid: int, albumid: int | None = None
    ) -> list[LidarrRename]:
        """Get files matching specified id that are not properly renamed yet."""
        params = {ARTIST_ID: artistid}
        if albumid is not None:
            params[ALBUM_ID] = albumid
        return await self._async_request(
            "rename",
            params=params,
            datatype=LidarrRename,
        )

    async def async_get_manual_import(  # pylint: disable=too-many-arguments
        self,
        downloadid: str,
        artistid: int = 0,
        folder: str | None = None,
        filterexistingfiles: bool = True,
        replaceexistingfiles: bool = True,
    ) -> list[LidarrManualImport]:
        """Get manual import."""
        params = {
            ARTIST_ID: artistid,
            "downloadId": downloadid,
            "filterExistingFiles": str(filterexistingfiles),
            "folder": folder if folder is not None else "",
            "replaceExistingFiles": str(replaceexistingfiles),
        }
        return await self._async_request(
            "manualimport", params=params, datatype=LidarrManualImport
        )

    async def async_edit_manual_import(
        self, data: LidarrManualImport
    ) -> list[LidarrManualImport]:
        """Get manual import."""
        return await self._async_request(
            "manualimport",
            data=data,
            datatype=LidarrManualImport,
            method=HTTPMethod.PUT,
        )

    async def async_get_retag(
        self, artistid: int, albumid: int | None = None
    ) -> list[LidarrRetag]:
        """Get retag."""
        params = {ARTIST_ID: artistid}
        if albumid is not None:
            params[ALBUM_ID] = albumid
        return await self._async_request(
            "retag",
            params=params,
            datatype=LidarrRetag,
        )

    async def async_search(self, term: str) -> list[LidarrSearch]:
        """Search for albums."""
        return await self._async_request(
            "search",
            params={TERM: term},
            datatype=LidarrSearch,
        )

    async def async_get_tags_details(
        self, tagid: int | None = None
    ) -> LidarrTagDetails | list[LidarrTagDetails]:
        """Get information about tag details.

        id: Get tag details matching id. Leave blank for all.
        """
        return await self._async_request(
            f"tag/detail{'' if tagid is None else f'/{tagid}'}",
            datatype=LidarrTagDetails,
        )

    async def async_get_tracks(
        self,
        artistid: int | None = None,
        albumid: int | None = None,
        albumreleaseid: int | None = None,
        trackids: int | list[int] | None = None,
    ) -> LidarrTrack | list[LidarrTrack]:
        """Get tracks based on specified ids.

        trackids: specify one integer to search database by track id only
        """
        if (
            artistid is None
            and albumid is None
            and albumreleaseid is None
            and trackids is None
        ):
            msg = "BadRequest: artistId, albumId, albumReleaseId or trackIds must be provided"
            raise ArrException(message=msg)
        params: dict[str, int | list[int]] = {}
        if artistid is not None:
            params[ARTIST_ID] = artistid
        if albumid is not None:
            params[ALBUM_ID] = albumid
        if albumreleaseid is not None:
            params["albumReleaseId"] = albumreleaseid
        if isinstance(trackids, list):
            params["trackIds"] = trackids
        return await self._async_request(
            f"track{f'/{trackids}' if isinstance(trackids, int) else ''}",
            params=params,
            datatype=LidarrTrack,
        )

    async def async_get_track_files(
        self,
        artistid: int | None = None,
        albumid: int | None = None,
        trackfileids: int | list[int] | None = None,
        unmapped: bool = False,
    ) -> LidarrTrackFile | list[LidarrTrackFile]:
        """Get track files based on specified ids.

        trackfileids: specify one integer to include audioTags for that id
        unmapped: True to instead get all files not matched to known albums
        """
        if artistid is None and albumid is None and trackfileids is None:
            raise ArrException(
                message="BadRequest: artistId, albumId, trackFileIds or unmapped must be provided"
            )
        params: dict[str, str | int | list[int]] = {"unmapped": str(unmapped)}
        if artistid is not None:
            params[ARTIST_ID] = artistid
        if albumid is not None:
            params[ALBUM_ID] = albumid
        if isinstance(trackfileids, list):
            params["trackFileIds"] = trackfileids
        return await self._async_request(
            f"trackfile{f'/{trackfileids}' if isinstance(trackfileids, int) else ''}",
            params=params,
            datatype=LidarrTrackFile,
        )

    async def async_edit_track_files(
        self, data: LidarrTrackFile | LidarrTrackFileEditor
    ) -> LidarrTrackFile | list[LidarrTrackFile]:
        """Edit track file attributes."""
        return await self._async_request(
            f"trackfile{'' if isinstance(data, LidarrTrackFile) else '/editor'}",
            data=data,
            datatype=LidarrTrackFile,
            method=HTTPMethod.PUT,
        )

    async def async_delete_track_files(self, ids: int | list[int]) -> None:
        """Delete track files. Use integer for one file or list for mass deletion."""
        return await self._async_request(
            f"trackfile/{'bulk' if isinstance(ids, list) else f'{ids}'}",
            data={"trackFileIds": ids} if isinstance(ids, list) else None,
            method=HTTPMethod.DELETE,
        )

    async def async_get_languages(self, langid: int | None = None) -> Any:
        """Get language profiles."""
        raise NotImplementedError()
